/* global jQuery, adamaSettings, Modernizr, google */
(function( $ ) {
	"use strict";

	var $window = $( window ),
		$document = $( document ),
		$body = $( document.body ),
		$wrapper = $body.find( '> .wrapper' ),
		$wpadminbar,
		$navbar,
		windowLastWidth = $window.width(),
		heightAdminBarLast = -1,
		heightStickyLast = -1,
		sections = [],
		disableOnePageScroll = false,
		isStickyEnabled = false;

	/**
	 * Helper function - restrict integer range and set default value in case number is not a number
	 *
	 * @param {int} value
	 * @param {int|bool} min
	 * @param {int|bool} max
	 * @param {int} defaultValue
	 * @returns {int}
	 */
	function intRangeDefault( value, min, max, defaultValue ) {
		value = parseInt( value, 10 );

		if ( isNaN( value ) ) {
			value = defaultValue;
		} else if ( min !== false && value < min ) {
			value = min;
		} else if ( max !== false && value > max ) {
			value = max;
		}

		return value;
	}

	/**
	 * Helper function - parse bool value and set default bool value in case one was not set
	 *
	 * @param {string} value
	 * @param {bool} defaultValue
	 * @returns {bool}
	 */
	function boolDefault( value, defaultValue ) {

		if ( typeof value === 'boolean' ) {
			return value;
		} else if ( typeof value === 'string' ) {
			value = value.toLowerCase();
		}

		if ( value === '1' || value === 'true' || value === 'yes' ) {
			value = true;
		} else if ( value === '0' || value === 'false' || value === 'no' ) {
			value = false;
		} else {
			value = defaultValue;
		}

		return value;
	}

	/**
	 * Helper function - Debounce events
	 * http://stackoverflow.com/a/24004942/111433
	 *
	 * @param {Function} func
	 * @param {int} wait
	 * @param {bool} immediate
	 * @returns {Function}
	 */
	function debounce( func, wait, immediate ) {
		var timeout;
		return function() {
			var context = this,
				args = arguments;

			var callNow = immediate && !timeout;

			clearTimeout( timeout );
			timeout = setTimeout( function() {
				timeout = null;

				// Check if the function already ran with the immediate flag
				if ( !immediate ) {
					// Call the original function with apply apply lets you define the 'this' object as well as the arguments (both captured before setTimeout)
					func.apply( context, args );
				}
			}, wait );

			if ( callNow ) {
				func.apply( context, args );
			}
		};
	}

	/**
	 * Initialize Google map
	 *
	 * @param {object} geocoder
	 * @param {object} infoWindow
	 * @param {object} map
	 * @param {bool} mapCentered
	 * @param {string} address
	 * @param {string} tooltip
	 * @returns {void}
	 */
	function googleMapInit( geocoder, infoWindow, map, mapCentered, address, tooltip ) {
		geocoder.geocode( { address: address }, function( results, status ) {
			var marker;

			if ( status === google.maps.GeocoderStatus.OK ) {

				if ( !mapCentered ) {
					map.setCenter( results[0].geometry.location );
					mapCentered = true;
				}

				marker = new google.maps.Marker({
					map: map,
					position: results[0].geometry.location
				});

				google.maps.event.addListener( marker, 'click', function() {
					infoWindow.setContent( tooltip );
					infoWindow.open( map, marker );
				} );

			} else {
				if ( window.console ) {
					console.log( 'Adama', adamaSettings.tErrorGeocode + status );
				}
			}
		});
	}

	/**
	 * Set height of tallest block to each element in the same "row".
	 *
	 * @param {jQuery} $elements
	 * @returns {void}
	 */
	function equalHeight( $elements ) {

		// Do we have any elements to resize
		if ( !$elements.length ) {
			return;
		}

		var groups = {},
			key;

		// Group elements by current top position
		$elements.each( function() {
			var $this = $( this ),
				height = $this.height( 'auto' ).outerHeight( false ),
				top = Math.round( this.getBoundingClientRect().top ),
				key = 't' + top;

			if ( typeof groups[key] === 'undefined' ) {
				groups[key] = {
					maxHeight: height,
					elements: [$this]
				};
			} else {
				if ( height > groups[key].maxHeight ) {
					groups[key].maxHeight = height;
				}

				groups[key].elements.push($this);
			}
		} );

		// Set max heights for each elements in a "row"
		for ( key in groups ) {
			if ( groups.hasOwnProperty( key ) ) {

				// Set height only if there are two or more elements in a row
				if ( groups[key].elements.length > 1 ) {
					for ( var i = 0; i < groups[key].elements.length; i++ ) {
						groups[key].elements[i].css( 'height', groups[key].maxHeight + 'px' );
					}
				}

			}
		}
	}

	/**
	 * Validate forms
	 *
	 * @param {object} e
	 * @returns {void}
	 */
	function validateForm( e ) {
		/*jshint validthis:true */

		var $form = $( this ),
			hasError = false;

		// Remove old error messages
		$form.find( '.alert' ).remove();

		// Validate
		$form.find( '[required]' ).each( function() {
			var $ctrl = $( this ),
				value = $.trim( $ctrl.val() ),
				message;

			if ( value === '' ) {
				hasError = true;
				message = adamaSettings.tErrorRequiredField;
			} else if ( $ctrl.attr( 'id' ) === 'email' && !( /^([\w-\+\.]+@([\w-]+\.)+[\w-]{2,4})?$/.test( value ) ) ) {
				hasError = true;
				message = adamaSettings.tErrorInvalidEmail;
			}

			if ( message ) {
				$ctrl.parent().append( '<div class="alert alert-danger alert-form-message">' + message + '</div>' );
			}
		});

		if ( hasError ) {
			$form.prepend( '<div class="alert alert-danger">' + adamaSettings.tErrorGlobal + '<button type="button" class="close" data-dismiss="alert" aria-hidden="true">&times;</button></div>' );
			e.preventDefault();
		}
	}

	/**
	 * Toggle shortcode content visibility.
	 *
	 * @returns {void}
	 */
	function shortcodeToggle() {
		/*jshint validthis:true */

		var $toggle = $( this ).next();

		$toggle.slideToggle( 'fast', function() {
			if ( $toggle.is( ':visible' ) ) {
				$toggle.parent().addClass( 'active' );
			} else {
				$toggle.parent().removeClass( 'active' );
			}
		} );
	}

	/**
	 * Add .animate-hover-slide-small to small elements.
	 *
	 * @returns {void}
	 */
	function animateHoverSmall() {
		var $elements = $( '.animate-hover-slide' );

		$elements.each( function() {
			var $this   = $( this ),
				isSmall = $this.hasClass( 'animate-hover-slide-small' ),
				width   = $this.find( '.figure' ).width();

			if ( !isSmall && ( width <= 200 )  ) {
				$this.addClass( 'animate-hover-slide-small' );
			} else if ( isSmall && ( width > 200 ) ) {
				$this.removeClass( 'animate-hover-slide-small' );
			}
		} );
	}

	/**
	 * Set positions for store demo notice and sticky header
	 *
	 * @returns {void}
	 */
	function setHeaderPositions() {
		var heightAdminBar = 0,
			heightSticky = 0;

		// Calculate height - Admin bar
		if ( $wpadminbar && $wpadminbar.length ) {
			heightAdminBar = $wpadminbar.outerHeight( false );
		}

		// Sticky header
		if ( isStickyEnabled ) {
			heightSticky = $navbar.outerHeight( false );

			if ( heightAdminBar !== heightAdminBarLast || heightSticky !== heightStickyLast ) {
				$navbar.unstick();

				if ( windowLastWidth >= adamaSettings.mobileMenuBreakpoint ) {
					$navbar.sticky( {
						topSpacing: heightAdminBar,
						className : 'navbar-fixed'
					} );
				}
			}

			heightStickyLast = heightSticky;
		}

		heightAdminBarLast = heightAdminBar;
	}

	/**
	 * Set positions for Mega menus
	 *
	 * @returns {void}
	 */
	function setMegaMenusPositions() {
		if ( !$navbar ) {
			return;
		}

		var $megaMenuItems = $navbar.find( '.galactica-default-navbar .mega-menu' );

		if ( $megaMenuItems.length > 0 ) {
			$megaMenuItems.each( function() {
				var $item = $( this ),
					$megaMenu = $item.find( '> .sub-menu-container' ),
					$itemPosition = $item.position(),
					$megaMenuNewPosition = $itemPosition.top + $item.height();

					if ( $megaMenu.length > 0 ) {
						$megaMenu.css( { top: $megaMenuNewPosition } );
					}
			} );
		}
	}

	/**
	 * Show uncover footer
	 *
	 * @returns {void}
	 */
	function footerUncover() {
		if ( adamaSettings.footerEnableUncover === '1' ) {
			var $footerElement = $( '.main-footer' ),
				$mainContent = $( '#main-content' ),
				$wrapper = $( '.wrapper' ),
				footerHeight = $footerElement.outerHeight( false ) - 1; // -1 to compensate for decimal places in certain situations

			if ( $window.innerHeight() > footerHeight * 1.2 ) {
				$mainContent.css( { 'margin-bottom': footerHeight + 'px' } );
				$wrapper.css( { 'min-height': $window.innerHeight() - footerHeight - $wrapper.offset().top + 'px' } );

				if ( !$footerElement.hasClass( 'footer-uncover' ) ) {
					$footerElement.addClass( 'footer-uncover' );
				}
			} else {
				$mainContent.css( { 'margin-bottom': 0 } );
				$wrapper.css( { 'min-height': 0 } );

				$footerElement.removeClass( 'footer-uncover' );
			}
		}
	}

	/**
	 * Init LightBox
	 *
	 * @returns {void}
	 */
	function zoomLightBox() {
		try {
			$( 'a.zoom' ).iLightBox( {
				skin: adamaSettings.lightboxSkin || 'dark',
				path: adamaSettings.lightboxDirection,
				controls: {
					arrows: boolDefault( adamaSettings.lightboxShowArrows, true ),
					thumbnail: boolDefault( adamaSettings.lightboxShowThumnails, false )
				}
			} );
		} catch ( ex ) {
			// Catch errors because of situations where there is no image available
			if ( window.console ) {
				console.error( 'zoomLightBox error:', ex.message );
			}
		}
	}

	var isInfiniteScrollAjaxRunning = false,
		infiniteLoaderErrorCount = 0;
	/**
	 * Infinite loader
	 *
	 * @param {object} e
	 * @returns {void}
	 */
	$body.on( 'click', '.pagination-infinite a', function( e ) {
		e.preventDefault();

		var $this = $( this ),
			$paginationContainer = $this.parent(),
			$container = $( '.mixitup' ).first(),
			$postPaged = parseInt( $container.data( 'current-page' ), 10 ),
			$currentTerm = $( '.ajax-filter' ).first().find( '.filter-ajax.active' );

		var data = {
				security:   adamaSettings.nonceAjax,
				action:     'galactica_infinite_loader',
				type:       'portfolio',
				pageID:     adamaSettings.pageID,
				paged:      $postPaged,
				targetPage: $postPaged + 1,
				// postsCount: $container.children( '.mix' ).length,
				categoryId: $currentTerm.data( 'category-id' ) || ''
			};

		$.ajax( {
			type: "POST",
			url:  adamaSettings.ajaxurl,
			data: data,
			beforeSend: function() {
				isInfiniteScrollAjaxRunning = true;
				$paginationContainer.show();
			},
			success: function( response ) {
				var errorMessage = '';
				if ( typeof response !== 'object' || !response.status ) {
					errorMessage = adamaSettings.tErrorAjaxBadData;
				} else if ( response.status === 'error' ) {
					errorMessage = response.message;
				}

				if ( errorMessage ) {
					infiniteLoaderErrorCount++;

					if ( window.console ) {
						console.log( 'Adama', errorMessage );
					}
					return;
				}


				var totalElements = $container[0].config.origOrder.length;

				// Add new elements(s) after last target element (subtract 1 as .eq() is zero-based):
				$container.find( '.mix' ).eq( totalElements - 1 ).after( response.html );

				var selectedFilter = $currentTerm.data( 'filter' ) ? $currentTerm.data( 'filter' ) : 'all';

				// Perform a remix and fade-in new elements
				$container.mixitup( 'remix', selectedFilter );

				// Update current page field
				$container.data( 'current-page', response.currentPage );

				if ( response.nextPage === 0 ) {
					$paginationContainer.remove();
				}

				// Reset error counter
				infiniteLoaderErrorCount = 0;
			},
			complete: function() {
				$paginationContainer.hide();
				isInfiniteScrollAjaxRunning = false;
			}
		} );
	} );

	/**
	 * Infinite scroll load more
	 *
	 * @returns {void}
	 */
	function infiniteScrollLoadMore() {
		if ( isInfiniteScrollAjaxRunning ) {
			return;
		} else if ( infiniteLoaderErrorCount >= 10 ) {
			if ( window.console ) {
				console.log( 'Adama', adamaSettings.tErrorAjaxAborting );
			}
			return;
		}

		var $button = $( '.pagination-infinite a' );
		if ( $button.length ) {
			var buttonOffset = $button.offset();
			if ( buttonOffset && $window.scrollTop() > ( buttonOffset.top - $window.height() ) / 2 ) {
				$button.trigger( 'click' );
			}
		}
	}

	/**
	 * Filter portfolio items via ajax
	 *
	 * @param {object} e
	 * @returns {void}
	 */
	$( '.ajax-filter' ).on( 'click', '.filter-ajax', function( e ) {
		e.preventDefault();

		var $thisFilter = $( this ),
			$container = $( '.mixitup' ).first(),
			$paginationContainer = $container.find( '.pagination-infinite' );


		// Mark all inactive
		$thisFilter.siblings( '.active' ).removeClass( 'active' );

		// Remove current items
		$container.children( '.mix' ).remove();


		var data = {
			security:   adamaSettings.nonceAjax,
			action:     'galactica_infinite_loader',
			type:       'portfolio',
			pageID:     adamaSettings.pageID,
			paged:      0,
			targetPage: 1,
			categoryId: $thisFilter.data( 'category-id' ) || ''
		};

		$.ajax( {
			type: "POST",
			url:  adamaSettings.ajaxurl,
			data: data,
			beforeSend: function() {
				isInfiniteScrollAjaxRunning = true;

				if ( $paginationContainer.length ) {
					$paginationContainer.show();
				} else {
					$paginationContainer = $( '<div class="pagination-infinite"><div class="infinite-loader"></div><a href="#">' + adamaSettings.tInfiniteScrollLoading + '</a></div>' );
					$container.append( $paginationContainer );
				}
			},
			success: function( response ) {
				var errorMessage = '';
				if ( typeof response !== 'object' || !response.status ) {
					errorMessage = adamaSettings.tErrorAjaxBadData;
				} else if ( response.status === 'error' ) {
					errorMessage = response.message;
				}

				if ( errorMessage ) {
					infiniteLoaderErrorCount++;

					if ( window.console ) {
						console.log( 'Adama', errorMessage );
					}
					return;
				}


				// Add new elements
				$container.prepend( response.html );

				var selectedFilter = $thisFilter.data( 'filter' ) ? $thisFilter.data( 'filter' ) : 'all';

				// Perform a remix and fade-in new elements
				$container.mixitup( 'remix', selectedFilter );

				// Update current page field
				$container.data( 'current-page', response.currentPage );

				if ( response.nextPage === 0 ) {
					$paginationContainer.remove();
				}

				// Mark current filter button active
				$thisFilter.addClass( 'active' );

				// Reset error counter
				infiniteLoaderErrorCount = 0;
			},
			complete: function() {
				$paginationContainer.hide();
				isInfiniteScrollAjaxRunning = false;
			}
		} );

	} );

	/**
	 * Toggle mobile menu
	 *
	 * @param {object} e
	 * @returns {void}
	 */
	$body.on( 'click', '.navbar-toggle-menu', function( e ) {
		e.preventDefault();

		toggleNav();
	} );

	/**
	 * Toggle mobile menu
	 *
	 * @returns {void}
	 */
	function toggleNav() {
		var $menu = $( '.galactica-mobile-navbar > .galactica-nav' );

		if ( $menu.length > 0 ) {
			// Open current menu ancestors
			$menu.find( '.current-menu-ancestor' ).addClass( 'active' ).children( '.sub-menu-container, .sub-menu-container-visible' ).show();

			if ( !$menu.is( ':visible' ) ) {
				$menu.slideDown( 'fast' );
			} else {
				$menu.slideUp( 'fast' );
			}
		}
	}

	/**
	 * Toggle mobile submenu
	 *
	 * @param {object} e
	 * @returns {void}
	 */
	$body.on( 'click', '.toggle-submenu', function( e ) {
		e.preventDefault();

		var $toggleParent = $( this ).parent( 'li' ),
			$activeChildren;

		if ( !$toggleParent.hasClass( 'active' ) ) {
			var $activeSiblings = $toggleParent.siblings( '.active' );

			$activeSiblings
				.removeClass( 'active' )
				.find( '.sub-menu-container, .sub-menu-container-visible' )
				.slideUp( 'fast' );

			$activeChildren = $activeSiblings.find( '.active' );

			$toggleParent.addClass( 'active' ).children( '.sub-menu-container, .sub-menu-container-visible' ).slideDown( 'fast' );
		} else {
			$activeChildren = $toggleParent.find( '.active' );

			$toggleParent.removeClass( 'active' ).find( '.sub-menu-container, .sub-menu-container-visible' ).slideUp( 'fast' );
		}

		$activeChildren.removeClass( 'active' );
	} );

	/**
	 * Set active link on scroll
	 *
	 * @returns {void}
	 */
	function onePageScroll() {
		var windowTop = $window.scrollTop(),
			sectionId = null,
			windowHeight = $window.height();

		if ( ( windowTop + windowHeight ) < $document.height() ) {
			for ( var i = 0; i < sections.length; i++ ) {
				if ( sections[i].sectionOffset < windowTop ) {
					sectionId = sections[i].linkHref;
				}
			}
		} else if ( sections.length > 0 ) {
			sectionId = sections[sections.length - 1].linkHref;
		}

		if ( sectionId !== null ) {
			var $parent = $navbar.find( 'a[href$="#' + sectionId + '"]' ).parent();

			// If it's not already the current section
			if ( !$parent.hasClass( 'current-one-page-item' ) ) {
				// Change the current menu item
				$navbar.find( '.current-one-page-item' ).removeClass( 'current-one-page-item' );
				$parent.addClass( 'current-one-page-item' );
			}
		} else {
			$navbar.find( '.current-one-page-item' ).removeClass( 'current-one-page-item' );
		}
	}

	/**
	 * One Page Navigation - calculate section offset
	 *
	 * @param {object} $section
	 * @returns {int}
	 */
	function onePageCalcSectionOffset( $section ) {
		var offsetTop = 25;

		if ( windowLastWidth > 585 ) {
			offsetTop += heightAdminBarLast;

			if ( isStickyEnabled && ( windowLastWidth >= adamaSettings.mobileMenuBreakpoint ) ) {
				offsetTop += heightStickyLast;
			}
		}

		var $mobileNav = $( '.galactica-mobile-navbar > .galactica-nav' );
		if ( $mobileNav.css('display') !== 'none' ) {
			offsetTop += $mobileNav.height();
		}

		return Math.round( $section.offset().top - offsetTop );
	}

	/**
	 * Calculate heights
	 *
	 * @returns {void}
	 */
	function calculateHeight() {
		for ( var i = 0; i < adamaSettings.equalHeight.length; i++ ) {
			equalHeight( $( adamaSettings.equalHeight[i].selector ) );
		}
	}

	/**
	 * Recalculate heights after carosuel resize
	 *
	 * @returns {void}
	 */
	function equalHeightSliderResized() {
		/*jshint validthis:true */
		var $this = $( this ),
			dataSelectors = $this.data( 'equal' );

		for ( var i = 0; i < dataSelectors.length; i++ ) {
			equalHeight( $this.find( dataSelectors[i] ) );
		}
	}

	/**
	 * Reposition masonry after slider resize
	 *
	 * @returns {void}
	 */
	function masonrySliderResized() {
		/*jshint validthis:true */
		$( this ).closest( '.post-entries' ).masonry( 'layout' );
	}

	/**
	 * Handle resize event
	 *
	 * @returns {void}
	 */
	function onResize() {

		var width = $window.width();

		// Only on width resize
		if ( width !== windowLastWidth ) {

			// Save new window width
			windowLastWidth = width;

			// Recalculate header positions
			setHeaderPositions();

			// Recalculate mega menus positions
			setMegaMenusPositions();

			// Recalculate heights
			calculateHeight();

			// Responsive animate hover
			animateHoverSmall();

		}

		// Footer uncover
		footerUncover();
	}

	/**
	 * Handle window scroll events
	 *
	 * @returns {void}
	 */
	function onScroll() {

		if ( $.isReady && !disableOnePageScroll ) {
			// One page navigation - scroll
			onePageScroll();
		}

		// Handle infinite scroll
		infiniteScrollLoadMore();

	}

	/**
	 * On DOM ready
	 *
	 * @returns {void}
	 */
	$document.ready( function() {

		// Calculate positions for header elements
		$wpadminbar = $( '#wpadminbar' );
		$navbar = $( '.navbar-primary' );
		isStickyEnabled = $.fn.sticky && $navbar && $body.hasClass( 'sticky-header' );
		setHeaderPositions();


		// On window resize (debounced)
		var debounceOnResize = debounce( onResize, 128, false );
		$window.resize( debounceOnResize );

		// On window scroll
		$window.on( 'scroll', onScroll );


		var navTimeoutShow,
			navTimeoutHide;

		$body.on( 'mouseenter', '.galactica-default-navbar .menu-item-has-children', function() {
			var $this = $( this );

			$this.addClass( 'galactica-hover' );

			clearTimeout( navTimeoutShow );

			navTimeoutShow = setTimeout( function() {
				if ( $this.hasClass( 'galactica-hover' ) ) {
					$this.children( '.sub-menu-container' ).show().children( '.sub-menu' ).stop().css( 'visibility', 'visible' ).animate({
						'opacity': 1
					}, 300);
				}
			}, 1);
		} );

		$body.on( 'mouseleave', '.galactica-default-navbar .menu-item-has-children', function() {
			var $this = $( this );

			$this.removeClass( 'galactica-hover' );

			clearTimeout( navTimeoutHide );

			navTimeoutHide = setTimeout( function() {
				if ( !$this.hasClass( 'galactica-hover' ) ) {
					$this.children('.sub-menu-container').children( '.sub-menu' ).stop().animate({
						'opacity': 0
					}, 100, function() {
						$( this ).css( 'visibility', 'hidden' );
						$this.children('.sub-menu-container').hide();
					});
				}
			}, 1);
		} );

		// Detect if dropdown navigation would go off screen and reposition it
		$body.on( 'mouseenter mouseleave', '.galactica-default-navbar .menu-item-has-children', function() {
			var $subMenu = $( this ).find( '> .sub-menu-container > .sub-menu' );

			if ( $subMenu.length > 0 ) {
				if ( ( $wrapper.width() - $subMenu.offset().left - $wrapper.offset().left - $subMenu.width() ) < 0 ) {
					$subMenu.addClass( 'off-left' );
				} else if ( $subMenu.hasClass( 'off-left' ) ) {
					$subMenu.removeClass( 'off-left' );
				}
			}
		} );

		// Inview animations
		if ( $.fn.waypoint ) {
			$( '.pie-chart-in-view' ).waypoint( {
				context    : window,
				continuous : true,
				enabled    : true,
				horizontal : false,
				//offset     : 'bottom-in-view',
				//offset     : '90%',
				offset     : function() {
					var windowHeight = $window.height(),
						positionMin = windowHeight - 100,
						positionHalf = windowHeight - this.clientHeight / 2;

					return positionMin < positionHalf ? positionMin : positionHalf;
				},
				triggerOnce: true,
				handler    : function( direction ) {
					$( '.pie-chart', this ).each( function() {
						var $element = $( this ),
							colorEmpty = $element.data( 'color_empty' ),
							colorFill = $element.data( 'color_fill' );

						$element.easyPieChart( {
							barColor  : ( colorFill && colorFill !== '' ) ? colorFill : '#616161',
							trackColor: ( colorEmpty && colorEmpty !== '' ) ? colorEmpty : '#dddddd',
							scaleColor: false,
							lineCap   : 'round',
							lineWidth : intRangeDefault( $element.data( 'line_width' ), 1, 100, 10 ),
							size      : intRangeDefault( $element.data( 'size' ), 10, false, 200 ),
							easing    : 'easeOutCubic',
							onStep    : function( from, to, percent ) {
								$( this.el ).find( '.percent' ).text( Math.round( percent ) );
							},
							animate: {
								duration: 2000,
								enabled: true
							}
						} );
					} );
				}
			} );

			$( '.animate-in-view' ).waypoint( {
				context    : window,
				continuous : true,
				enabled    : true,
				horizontal : false,
				//offset     : 'bottom-in-view',
				//offset     : '90%',
				offset     : function() {
					var windowHeight = $window.height(),
						positionMin = windowHeight - 100,
						positionHalf = windowHeight - this.clientHeight / 2;

					return positionMin < positionHalf ? positionMin : positionHalf;
				},
				triggerOnce: true,
				handler    : function( direction ) {
					var $element = $( this ),
						animation = $element.data( 'animation' ),
						animationSpeed = $element.data( 'animation-speed' );

					if ( typeof animation !== 'undefined' && animation !== '' ) {

						if ( typeof animationSpeed !== 'undefined' && animationSpeed !== '' ) {
							animationSpeed += 's';
							$element.css( {
								'-moz-animation-duration': animationSpeed,
								'-ms-animation-duration': animationSpeed,
								'-o-animation-duration': animationSpeed,
								'animation-duration': animationSpeed
							} );
						}

						$element.one( 'webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function() {
							$element.removeClass( 'animated ' + animation );
							if ( animation.indexOf( 'Out' ) !== -1 ) {
								$element.css( { opacity: 0 } );
							}
						} );
						$element
							.addClass( 'animated ' + animation )
							.removeClass( 'galactica-animate-hide' );

					}
				}
			} );

			$( '.shortcode-popup-trigger-in-view' ).waypoint( {
				context    : window,
				continuous : true,
				enabled    : true,
				horizontal : false,
				//offset     : 'bottom-in-view',
				//offset     : '90%',
				offset     : function() {
					var windowHeight = $window.height(),
						positionMin = windowHeight - 100,
						positionHalf = windowHeight - this.clientHeight / 2;

					return positionMin < positionHalf ? positionMin : positionHalf;
				},
				triggerOnce: true,
				handler    : function( direction ) {
					var $this  = $( this ),
						target = $this.data( 'target' );

					if ( $.fn.modal && $this.data( 'popup-type' ) === 'bootstrap' ) {
						// Bootstrap modal popup
						$( target ).modal( 'show' );
					} else if ( $.fn.iLightBox ) {
						// iLightBox popup
						$.iLightBox( [{
							skin   : adamaSettings.lightboxSkin || 'dark',
							type   : $this.data( 'type' ) || 'inline',
							url    : target,
							title  : $this.data( 'title' ),
							options: {
								width   : $this.data( 'width' ),
								height  : $this.data( 'height' ),
								onRender: function( iL ) {
									iL.element[0].className += ' lightbox-popup-wrap';
								}
							}
						}] );
					}
				}
			} );
		}

		// Start Owl carousels
		if ( $.fn.owlCarousel ) {
			$( '.owl-carousel' ).each( function() {
				var $element = $( this ),
					items = intRangeDefault( $element.data( 'items' ), 1, false, 4 ),
					settings = {
						items             : items,
						loop              : boolDefault( $element.data( 'loop' ), false ),
						center            : boolDefault( $element.data( 'center' ), false ),
						margin            : intRangeDefault( $element.data( 'margin' ), 0, false, 10 ),
						startPosition     : intRangeDefault( $element.data( 'start' ), 0, false, 0 ),
						nav               : boolDefault( $element.data( 'show_nav' ), true ),
						dots              : boolDefault( $element.data( 'show_indicators' ), false ),
						autoplay          : boolDefault( $element.data( 'autoplay' ), true ),
						autoplayTimeout   : intRangeDefault( $element.data( 'interval' ), 100, false, 3000 ),
						autoplayHoverPause: boolDefault( $element.data( 'pause' ), true ),
						navText           : [ '', '' ],
						responsive : {
							// Breakpoint from 0 up
							0 : {
								items : 1
							},
							// Breakpoint from 768 up
							768 : {
								items : ( items > 1 ) ? 2 : 1
							},
							// Breakpoint from 992 up
							992 : {
								items : items
							}
						}
					};

				$element.owlCarousel( settings );
			});
		}

		// Validate contact forms
		$body.on( 'submit', '.comment-form', validateForm );

		// Toggle shortcode
		$body.on( 'click', '.shortcode-toggle-trigger', shortcodeToggle );

		// Init Google Maps
		if ( typeof google === 'object' && typeof google.maps === 'object' ) {
			$( '.google-map' ).each( function() {
				var $element = $( this ),
					addresses = $element.data( 'address' ),
					tooltips = $element.data( 'tooltip' ),
					tooltip,
					geocoder = new google.maps.Geocoder(),
					infoWindow = new google.maps.InfoWindow(),
					mapCentered = false,
					map = new google.maps.Map( this, {
						mapTypeId  : google.maps.MapTypeId[$element.data( 'type' ).toUpperCase()],
						zoom       : intRangeDefault( $element.data( 'zoom' ), 1, 20, 15 ),
						scrollwheel: boolDefault( $element.data( 'scrollwheel_enabled' ), false )
					} );

				if ( !addresses ) {
					return;
				}

				addresses = addresses.split( '|' );

				if ( tooltips ) {
					tooltips = tooltips.split( '|' );
				}

				for ( var i = 0; i < addresses.length; i++ ) {
					tooltip = ( tooltips && tooltips[i] ) ? tooltips[i] : addresses[i];
					googleMapInit( geocoder, infoWindow, map, mapCentered, addresses[i], tooltip, i );
				}
			});
		}

		// Slideout Sidebar
		var $sidebarSlideout = $( '#sidebar-slideout' );
		if ( $sidebarSlideout.length ) {
			$body.on( 'click', '.toggle-sidebar-slideout', function( e ) {
				e.preventDefault();

				if ( $sidebarSlideout.is( ':visible' ) ) {
					if ( Modernizr && Modernizr.cssanimations ) {
						$sidebarSlideout.one( 'webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function() {
							$sidebarSlideout
								.hide()
								.removeClass( 'animated bounceOutRight' );
						} );

						$sidebarSlideout.addClass( 'animated bounceOutRight' );
					} else {
						$sidebarSlideout.hide();
					}
				} else {
					$sidebarSlideout.one( 'webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function() {
						$sidebarSlideout.removeClass( 'animated bounceInRight' );
					} );

					var top = $body.children( '.wrapper' ).position().top;
					$sidebarSlideout
						.css( { top: top } )
						.show()
						.addClass( 'animated bounceInRight' );
				}
			} );
		}

		// One page navigation
		$( '.galactica-nav .menu-item a[href*="#"]:not([href="#"])' ).each( function() {

			// Skip anchors on different domain or different pages
			if ( location.hostname === this.hostname && location.pathname.replace( /^\//, '' ) === this.pathname.replace( /^\//, '' ) ) {
				return;
			}

			var $this = $( this ),
				linkHref = $this.attr( 'href' ).split( '#' )[1],
				$target = $( '#' + linkHref );

			if ( $target.length ) {

				var isDuplicate = false;
				for ( var i = 0; i < sections.length; i++ ) {
					if ( sections[i].linkHref === linkHref ) {
						isDuplicate = true;
						break;
					}
				}

				if ( !isDuplicate ) {
					sections.push( {
						linkHref     : linkHref,
						sectionOffset: onePageCalcSectionOffset( $target )
					} );
				}
			}
		});

		sections.sort( function( a, b ) {
			return a.sectionOffset - b.sectionOffset;
		} );

		/**
		 * One page navigation - click
		 *
		 * @param {object} $this
		 * @param {object} e
		 * @returns {bool}
		 */
		function onePageNavigationClick( $this, e ) {
			var linkHref = $this.attr( 'href' ).split( '#' )[1],
				$target = $( '#' + linkHref );

			if ( $target.length ) {
				$navbar.find( '.current-one-page-item' ).removeClass( 'current-one-page-item' );
				$this.parent().addClass( 'current-one-page-item' );
				disableOnePageScroll = true;

				$('body, html').animate( {
					scrollTop: onePageCalcSectionOffset( $target )
				}, 1000, 'swing',
				function() {
					setTimeout( function() {
						disableOnePageScroll = false;
					}, 100 );
				} );

				e.preventDefault();
				return true;
			}

			return false;
		}

		// One page default navigation
		$body.on( 'click', '.galactica-default-navbar .menu-item a[href*="#"]:not([href="#"])', function( e ) {
			onePageNavigationClick( $( this ), e );
		} );

		// One page mobile navigation
		$body.on( 'click', '.galactica-mobile-navbar .menu-item a[href*="#"]:not([href="#"])', function( e ) {
			var $this = $( this );

			if ( onePageNavigationClick( $this, e ) ) {
				// Close menu
				toggleNav();
			}
		} );

		// One page navigation - set active link on scroll
		onePageScroll();

		// Scroll to top
		if ( $.fn.UItoTop && adamaSettings.footerEnableToTop === '1' ) {
			$().UItoTop( {
				easingType: 'easeOutQuart'
			});
		}

		// WooCommerce - Add to cart spinner
		$body.on( 'adding_to_cart', function( e, $thisbutton ) {
			var $product = $thisbutton.closest( '.product' );
			$product.find( '.add-to-cart-spinner > .fa-check-circle-o' ).removeClass( 'fa-check-circle-o' ).addClass( 'fa-spinner fa-spin' );

			// Show add to cart spinner
			$product.find( '.add-to-cart-spinner' ).show();
		});
		$body.on( 'added_to_cart', function( e, fragments, cart_hash, $thisbutton ) {
			// Hide add to cart spinner
			var $addToCartSpinner;

			// WooCommerce > v2.3.6
			if ( $thisbutton ) {
				$addToCartSpinner = $thisbutton.closest( '.product' ).find( '.add-to-cart-spinner' );

				$addToCartSpinner.children( '.fa-spinner' )
					.removeClass( 'fa-spinner fa-spin' )
					.addClass( 'fa-check-circle-o' );

				// Hide add to cart spinner
				$addToCartSpinner
					.fadeOut( 2000 );
			} else {
				var added = $( '.add-to-cart-hover .added' );
				for ( var i = 0; i < added.length; i++ ) {
					$addToCartSpinner = $( added[i] ).closest( '.product' ).find( '.add-to-cart-spinner' );
					$addToCartSpinner.children( '.fa-spinner' )
						.removeClass( 'fa-spinner fa-spin' )
						.addClass( 'fa-check-circle-o' );

					// Hide add to cart spinner
					$addToCartSpinner
						.fadeOut( 2000 );
				}
			}
		});

		// Tooltips init
		if ( $.fn.tooltip ) {
			$( '[data-toggle="tooltip"]' ).tooltip();
		}

		// MixItUp
		if ( $.fn.mixitup ) {
			$( '.mixitup' ).mixitup();
		}

		// Lightbox
		if ( $.fn.iLightBox ) {
			$body
				// Archive zoom
				.on( 'click', '.zoom-archive', function( e ) {
					e.preventDefault();

					var $this = $( this ),
						images = [{	url: $this.attr( 'href' ) }],
						imagesGallery = $this.data( 'images' ),
						controls = {};

					if ( imagesGallery ) {
						images = images.concat( imagesGallery );

						controls = {
							arrows: boolDefault( adamaSettings.lightboxShowArrows, true ),
							thumbnail: boolDefault( adamaSettings.lightboxShowThumnails, false )
						};
					}

					$.iLightBox( images, {
						skin: adamaSettings.lightboxSkin || 'dark',
						path: adamaSettings.lightboxDirection,
						controls: controls
					} );
				} )
				// Shortcode popup
				.on( 'click', '.shortcode-popup-trigger-click', function() {
					var $this = $( this );
					$.iLightBox( [{
						skin   : adamaSettings.lightboxSkin || 'dark',
						type   : $this.data( 'type' ) || 'inline',
						url    : $this.data( 'target' ),
						title  : $this.data( 'title' ),
						options: {
							width   : $this.data( 'width' ),
							height  : $this.data( 'height' ),
							onRender: function( iL ) {
								iL.element[0].className += ' lightbox-popup-wrap';
							}
						}
					}] );
				} );

			// Attach to all links that have img as direct child
			if ( boolDefault( adamaSettings.lightboxEnableForPosts, true ) ) {
				$( '.entry-content a:not(.zoom) > img' ).each( function() {

					// Try to detect if a.href is link to image, we don't need lightbox for page links
					if ( this.src && this.parentElement.href && this.src.slice( -4 ) === this.parentElement.href.slice( -4 ) ) {
						$( this ).parent()
							.addClass( 'zoom' );
//							.attr( 'data-rel', 'prettyPhoto[gallery]' );
					}

				} );
			}

			// WooCommerce v3.0.0 fix for zoom
			$( '.woocommerce-product-gallery__image > a' ).addClass( 'zoom' );

			zoomLightBox();
		}

		// Animations
		// NOTE: hover animations are done without JS
		$body.on( 'click', '.animate-click', function() {
			var $this = $( this ),
				$animatedElement = $this.siblings( '.animate-wr' ),
				animateIn = $this.data( 'animate-in' ),
				animateOut = $this.data( 'animate-out' ),
				animate = $animatedElement.is( ':hidden' ) ? animateIn : animateOut;

			if ( typeof animate !== 'undefined' && animate !== '' ) {
				$animatedElement.one( 'webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function() {
					$animatedElement.removeClass( animateIn + ' ' + animateOut );
				} );

				$animatedElement
					.removeClass( animateIn + ' ' + animateOut )
					.addClass( animate );
			}
		} );


		// Placeholder support for old browsers
		if ( $.placeholder && !$.placeholder.browser_supported() ) {
			$.placeholder.shim();

			$body.on( 'click', '.comment-reply-link, #cancel-comment-reply-link', function() {
				$.placeholder.shim();
			} );
		}
	} );

	/**
	 * On window load - Runs after all media is loaded
	 *
	 * @returns {void}
	 */
	$window.load( function() {
		// Recalculate heights
		calculateHeight();

		// Set positions for Mega menus
		setMegaMenusPositions();

		// Footer uncover
		footerUncover();

		// Responsive animate hover
		animateHoverSmall();

		// Handle infinite scroll
		infiniteScrollLoadMore();

		// Masonry
		if ( $.fn.masonry ) {
			// Reposition masonry after slider resize
			var debounceMasonrySliderResized = debounce( masonrySliderResized, 200, false );

			// Init masonry
			$( '.masonry .post-entries' )
				.masonry( {
					itemSelector: '.post'
				} )
				.on( 'resized.owl.carousel', '.owl-carousel', debounceMasonrySliderResized );
		}

		// Recalculate heights after carosuel resize
		for ( var i = 0; i < adamaSettings.equalHeight.length; i++ ) {
			if ( 'carousel' in adamaSettings.equalHeight[i] ) {
				var $carousel = $( adamaSettings.equalHeight[i].carousel ),
					dataSelectors = [];

				if ( !$carousel.length ) {
					continue;
				}

				if ( !$carousel.hasClass( 'carousel-equal' ) ) {
					$carousel.addClass( 'carousel-equal' );

					// NOTE: Debounce will not work if multiple carousels on page
					$carousel.on( 'resized.owl.carousel', equalHeightSliderResized );
				} else {
					dataSelectors = $carousel.data( 'equal' );
				}

				dataSelectors.push( adamaSettings.equalHeight[i].selector );
				$carousel.data( 'equal', dataSelectors );
			}
		}

		// Popup init
		$( '.shortcode-popup-trigger-load' ).each( function() {
			var $this  = $( this ),
				target = $this.data( 'target' ),
				delay  = parseInt( $this.data( 'trigger-delay' ) ) * 1000;

			window.setTimeout( function() {
				if ( $.fn.modal && $this.data( 'popup-type' ) === 'bootstrap' ) {
					// Bootstrap modal popup
					$( target ).modal( 'show' );
				} else if ( $.fn.iLightBox ) {
					// iLightBox popup
					$.iLightBox( [{
						skin   : adamaSettings.lightboxSkin || 'dark',
						type   : $this.data( 'type' ) || 'inline',
						url    : target,
						title  : $this.data( 'title' ),
						options: {
							width   : $this.data( 'width' ),
							height  : $this.data( 'height' ),
							onRender: function( iL ) {
								iL.element[0].className += ' lightbox-popup-wrap';
							}
						}
					}] );
				}
			}, delay );

		} );
	} );

})( jQuery );
