<?php
/**
 * Galactica Themes
 *
 * Breadcrumb functions.
 *
 * @package GalacticaThemes
 * @subpackage Framework
 */

// Don't allow direct access
if ( !defined( 'ABSPATH' ) ) {
	die;
}


/**
 * Main function for breadcrumbs.
 *
 * @param array $args Breadcrumb attributes
 * @return void
 */
function galactica_breadcrumb_builder( $args = array() ) {
	global $post, $wp_query;

	$defaults = array(
		'wrap_before'       => '<div class="breadcrumb hidden-xs">',
		'wrap_after'        => '</div>',
		'breadcrumb_before' => '<span class="%1$s">',
		'breadcrumb_after'  => '</span>',
		'home_link'         => esc_html_x( 'Home', 'breadcrumb', 'adama' ),
		'blog_link'         => true,
		'shop_link'         => true,
		'separator'         => '/',
		'separator_before'  => '<span class="separator">',
		'separator_after'   => '</span>',
		'on_front_page'     => true,
		'echo'              => true,
	);
	$args = wp_parse_args( apply_filters( 'galactica/breadcrumb/args', $args ), $defaults );

	$wrap_before       = $args['wrap_before'];
	$wrap_after        = $args['wrap_after'];
	$breadcrumb_before = $args['breadcrumb_before'];
	$breadcrumb_after  = $args['breadcrumb_after'];
	$home_link         = $args['home_link'];
	$blog_link         = $args['blog_link'];
	$shop_link         = $args['shop_link'];
	$separator         = $args['separator'];
	$separator_before  = $args['separator_before'];
	$separator_after   = $args['separator_after'];
	$on_front_page     = $args['on_front_page'];
	$echo              = $args['echo'];

	// Fetch page ID from WP options
	$page_for_posts = get_option( 'page_for_posts' );


	// Breadcrumb link trail
	$links = array();

	// Link to home page if $home_link is set
	if ( $home_link && !is_front_page() ) {
		$links[] = array( 'url' => home_url( '/' ), 'label' => $home_link );
	}

	// Add shop trail
	if ( $shop_link && function_exists( 'woocommerce_get_page_id' ) && (
		 ( function_exists( 'is_woocommerce' ) && is_woocommerce() ) ||
		 ( function_exists( 'is_cart' ) && is_cart() ) ||
		 ( function_exists( 'is_checkout' ) && is_checkout() ) ||
		 ( function_exists( 'is_account_page' ) && is_account_page() )
		) ) {

		$shop_id = wc_get_page_id( 'shop' );
		if ( !empty( $shop_id ) ) {
			$links[] = galactica_breadcrumb_get_page( $shop_id );
		}
	}


	if ( is_front_page() ) {
		// Front page

		if ( $home_link && $on_front_page ) {
			$links[] = array( 'url' => home_url( '/' ), 'label' => $home_link );
		}

	} else if ( is_home() ) {
		// Home/Posts page

		if ( $page_for_posts ) {
			galactica_breadcrumb_add_page_blog( $links, $page_for_posts );
		}

	} else if ( is_singular() ) {
		// Single page

		// Post Type Archive - skip for shop page
		galactica_breadcrumb_add_post_type_archive( $links, $post->post_type );

		if ( $post->post_parent ) {

			// Add parents if they exist
			galactica_breadcrumb_add_page_ancestors( $links, $post );

		} else {

			// Add category for supported post types
			$post_type = get_post_type();
			$post_type_categories_supported = apply_filters( 'galactica/breadcrumb/categories_supported', array(
				'bsg_faq'       => 'bsg_faq_category',
				'bsg_portfolio' => 'bsg_portfolio_category',

				'product'       => 'product_cat',
			) );

			if ( array_key_exists( $post_type, $post_type_categories_supported ) ) {
				$terms = wp_get_object_terms( $post->ID, $post_type_categories_supported[$post_type] );
				if ( is_array( $terms ) && !empty( $terms ) ) {

					// In case post has multiple categories use just the first one
					$term = reset( $terms );

					// Add term ancestors
					galactica_breadcrumb_add_term_ancestors( $links, $term, $post_type_categories_supported[$post_type] );

					// Add term
					$links[] = galactica_breadcrumb_get_term( $term );
				}
			}

		}

		$links[] = galactica_breadcrumb_get_page( $post->ID );

	} else if ( is_post_type_archive() ) {
		// Post type archives

		// Post Type Archive - skip for shop page
		galactica_breadcrumb_add_post_type_archive( $links, get_post_type() );

	} else if ( is_tax() || is_category() || is_tag() ) {
		// Taxonomies

		// Add Blog/Posts page
		if ( $blog_link && $page_for_posts && get_post_type() === 'post' ) {
			galactica_breadcrumb_add_page_blog( $links, $page_for_posts );
		}

		// Get current term
		$term = $wp_query->get_queried_object();

		// Add term ancestors
		galactica_breadcrumb_add_term_ancestors( $links, $term, $term->taxonomy );

		// Add term
		$links[] = galactica_breadcrumb_get_term( $term );

	} else if ( is_date() ) {

		// Add Blog/Posts page
		if ( $blog_link && $page_for_posts && get_post_type() === 'post' ) {
			galactica_breadcrumb_add_page_blog( $links, $page_for_posts );
		}

		if ( is_year() || is_month() || is_day() ) {
			$links[] = array( 'url' => get_year_link( get_the_time( 'Y' ) ), 'label' => get_the_time( esc_html_x( 'Y', 'breadcrumb', 'adama' ) ));
		}

		if ( is_month() || is_day() ) {
			$links[] = array( 'url' => get_month_link( get_the_time( 'Y' ), get_the_time( 'm' ) ), 'label' => get_the_time( esc_html_x( 'F', 'breadcrumb', 'adama' ) ));
		}

		if ( is_day() ) {
			$links[] = array( 'url' => get_day_link( get_the_time( 'Y' ), get_the_time( 'm' ), get_the_time( 'j' ) ), 'label' => get_the_time( esc_html_x( 'j', 'breadcrumb', 'adama' ) ));
		}

	} else if ( is_author() ) {

		// Fetch current author
		$user = $wp_query->get_queried_object();

		// Add author
		$links[] = array( 'url' => get_author_posts_url( $user->ID ), 'label' => sprintf( esc_html__( 'Author: %s', 'adama' ), $user->display_name ));

	} else if ( is_search() ) {

		// Search results
		$links[] = array( 'label' => sprintf( esc_html__( 'Search results for &ldquo;%s&rdquo;', 'adama' ), get_search_query() ));

	} else if ( is_404() ) {

		// Page not found
		$links[] = array( 'label' => esc_html__( 'Error 404 - Page Not Found', 'adama' ) );

	}



	// Allow child theme developers to change breadcrumb trail
	$links = apply_filters( 'galactica/breadcrumb/links', $links, $args );

	// Mark last element in a trail so we don't build link for it
	$last_link = array_pop( $links );
	if ( $last_link ) {
		$last_link['end'] = true;
		$links[] = $last_link;
	}

	// Build HTML
	$breadcrumb = '';
	foreach ( $links as $link ) {
		$is_active = !empty( $link['end'] );
		$is_link = !empty( $link['url'] ) && !$is_active;

		$breadcrumb_item = '';
		$breadcrumb_class = $is_active ? 'active' : '';

		// Separator
		if ( !empty( $breadcrumb ) ) {
			$breadcrumb_item .= $separator_before . $separator . $separator_after;
		}

		// Breadcrumb
		$breadcrumb_item .= sprintf( $breadcrumb_before, $breadcrumb_class );

		if ( $is_link ) {
			$breadcrumb_item .= '<a href="' . esc_url( $link['url'] ) . '">';
		}

		// Label
		$label = $link['label'];
		if ( str_word_count( $label, 0 ) > 8 ) {
			$label = wp_trim_words( $label, 5, '&hellip;' );
		}

		if ( empty( $label ) ) {
			$label = esc_html__( 'No Title', 'adama' );
		}
		$breadcrumb_item .= esc_html( $label );

		if ( $is_link ) {
			$breadcrumb_item .= '</a>';
		}

		$breadcrumb_item .= $breadcrumb_after;


		// Allow overriding single breadcrumb HTML
		$breadcrumb .= apply_filters( 'galactica/breadcrumb/links/item', $breadcrumb_item, $link, $args );
	}

	// Add Wrapper
	$breadcrumb = $wrap_before . $breadcrumb . $wrap_after;


	// Output breadcrumb
	if ( $echo ) {
		echo $breadcrumb;
		return true;
	} else {
		return $breadcrumb;
	}
}

/**
 * Adds single page to breadcrumb trail.
 *
 * @param int $id Page ID
 * @return array
 */
function galactica_breadcrumb_get_page( $id ) {
	return array( 'url' => get_permalink( $id ), 'label' => get_the_title( $id ) );
}

/**
 * Adds single term to breadcrumb trail.
 *
 * @param object $term Term
 * @return array
 */
function galactica_breadcrumb_get_term( $term ) {
	return array( 'url' => get_term_link( $term ), 'label' => $term->name );
}

/**
 * Tries to find page's ancestors and adds them to the breadcrumb trail
 *
 * @param array $links
 * @param object $post
 * @return void
 */
function galactica_breadcrumb_add_page_ancestors( &$links, $post ) {

	if ( $post->post_parent ) {

		// Fetch ancestors
		$post_ancestors = get_post_ancestors( $post );
		if ( isset( $post_ancestors ) ) {
			if ( is_array( $post_ancestors ) ) {
				$ancestors = array_values( $post_ancestors );
			} else {
				$ancestors = array($post_ancestors);
			}
		} else {
			$ancestors = array($post->post_parent);
		}

		// Reverse the order so it's oldest to newest
		$ancestors = array_reverse( $ancestors );

		foreach ( $ancestors as $ancestor_id ) {
			$links[] = galactica_breadcrumb_get_page( $ancestor_id );
		}
	}

}

/**
 * Add blog page
 *
 * @param array $links
 * @param int $page_id
 * @return void
 */
function galactica_breadcrumb_add_page_blog( &$links, $page_id ) {

	// Fetch page
	$page = get_post( $page_id );
	if ( isset( $page ) ) {

		// Add page ancestors if they exist
		galactica_breadcrumb_add_page_ancestors( $links, $page );

		// Add blog page
		$links[] = galactica_breadcrumb_get_page( $page->ID );
	}

}

/**
 * Checks if term is in hierarchical taxonomy, tries to find it's ancestors and adds them to the breadcrumb trail
 *
 * @param array $links
 * @param object $term
 * @param string $taxonomy
 * @return void
 */
function galactica_breadcrumb_add_term_ancestors( &$links, $term, $taxonomy ) {

	// Fetch ancestors
	// $ancestors = get_ancestors( $term->term_id, $post_type_categories_supported[$post_type] );
	$ancestors = array();
	if ( !empty( $term->parent ) && is_taxonomy_hierarchical( $taxonomy ) ) {
		//$term = get_term( $term->parent, $post_type_categories_supported[$post_type] );
		while ( !empty( $term->parent ) ) {
			$term = get_term( $term->parent, $taxonomy );
			if ( is_wp_error( $term ) ) {
				break;
			}

			$ancestors[] = $term;
		}
	}

	// Reverse the order so it's oldest to newest
	$ancestors = array_reverse( $ancestors );

	// Add ancestors
	foreach ( $ancestors as $ancestor ) {
		$links[] = galactica_breadcrumb_get_term( $ancestor );
	}

}

/**
 * Checks if post type has archive page enabled and adds it to the breadcrumb trail
 *
 * @param array $links
 * @param string $post_type
 * @return void
 */
function galactica_breadcrumb_add_post_type_archive( &$links, $post_type ) {

	// Skip for WooCommerce product archive
	if ( $post_type !== 'product' ) {

		$post_type_url = get_post_type_archive_link( $post_type );
		if ( $post_type_url ) {

			$post_type = get_post_type_object( $post_type );
			if ( isset( $post_type->label ) && $post_type->label !== '' ) {
				$post_type_title = $post_type->label;
			} else {
				$post_type_title = $post_type->labels->menu_name;
			}
			$links[] = array( 'url' => $post_type_url, 'label' => $post_type_title );
		}

	}

}
