<?php
/**
 * Galactica Themes
 *
 * Helper functions.
 *
 * @package GalacticaThemes
 * @subpackage Framework
 */

// Don't allow direct access
if ( !defined( 'ABSPATH' ) ) {
	die;
}


if ( !function_exists( 'galactica_option' ) ) {
	/**
	 * Return option from global variable.
	 *
	 * @param string $option Option ID
	 * @param mixed $default Default value to return in case option is not set
	 * @return mixed
	 */
	function galactica_option( $option, $default = false ) {
		return isset( $GLOBALS[GALACTICA_THEME_OPTIONS], $GLOBALS[GALACTICA_THEME_OPTIONS][$option] ) ? $GLOBALS[GALACTICA_THEME_OPTIONS][$option] : $default;
	}
}

if ( !function_exists( 'galactica_option_image_url' ) ) {
	/**
	 * Return image URL from option.
	 *
	 * @param string $option Option ID
	 * @param mixed $default Default value to return in case option is not set
	 * @return mixed
	 */
	function galactica_option_image_url( $option, $default = false ) {
		return ( !empty( $GLOBALS[GALACTICA_THEME_OPTIONS][$option] ) && !empty( $GLOBALS[GALACTICA_THEME_OPTIONS][$option]['url'] ) ) ? $GLOBALS[GALACTICA_THEME_OPTIONS][$option]['url'] : $default;
	}
}

if ( !function_exists( 'galactica_option_logo' ) ) {
	/**
	 * Return options for logo and logoX2.
	 * In case there is no logo but there is retina logo (x2), retina logo will be returned as logo.
	 *
	 * @return array
	 */
	function galactica_option_logo() {

		$defaults = array(
			'url'    => '',
			'width'  => '',
			'height' => '',
			'x2'     => array(
				'url'    => '',
				'width'  => '',
				'height' => '',
			),
		);

		$theme_options = $GLOBALS[GALACTICA_THEME_OPTIONS];
		$ret = isset( $theme_options['logo'] ) ? wp_parse_args( (array) $theme_options['logo'], $defaults ) : $defaults;

		if ( isset( $theme_options['logoX2'] ) ) {
			if ( empty( $ret['url'] ) ) {
				$data = &$ret;
			} else {
				$data = &$ret['x2'];
			}

			$data = wp_parse_args( (array) $theme_options['logoX2'], $defaults['x2'] );
			if ( !empty( $data['width'] ) ) {
				$data['width'] /= 2;
			}
			if ( !empty( $data['height'] ) ) {
				$data['height'] /= 2;
			}
		}

		return $ret;
	}
}

if ( !function_exists( 'galactica_option_email' ) ) {
	/**
	 * Return option with email address from global variable.
	 * If email option is not entered, admin email is used.
	 *
	 * @param string $option Option ID
	 * @return string
	 */
	function galactica_option_email( $option ) {
		return !empty( $GLOBALS[GALACTICA_THEME_OPTIONS][$option] ) ? $GLOBALS[GALACTICA_THEME_OPTIONS][$option] : get_option( 'admin_email', false );
	}
}

if ( !function_exists( 'galactica_portfolio_meta_option' ) ) {
	/**
	 * Return meta value if one the portfolio templates is active and if set otherwise option from global variable.
	 *
	 * @param string $key Meta key & Option ID
	 * @param mixed $default Default value to return in case option is not set
	 * @param int $page_id Page ID
	 * @return mixed
	 */
	function galactica_portfolio_meta_option( $key, $default = false, $page_id = null ) {

		if ( is_page_template( array( 'templates/portfolio-boxed.php', 'templates/portfolio-fluid.php' ) ) ) {
			if ( $page_id === null ) {
				// Current page ID
				$page_id = galactica_get_page_id();
			}

			if ( !empty( $page_id ) ) {
				// If page_id exists we are on single page
				$value = get_post_meta( $page_id, "adama_{$key}", true );
			}
		}

		if ( !isset( $value ) || $value === '' ) { // NOTE: do not use empty because we have valid '0' values
			// IF there is no value fetch from options
			$value = galactica_option( $key, $default );
		}

		return $value;
	}
}

if ( !function_exists( 'galactica_term_meta_option' ) ) {
	/**
	 * Return term meta value if WordPress 4.4+ and if set otherwise option from global variable.
	 *
	 * @param int $term_id Page ID
	 * @param string $prefix Meta prefix
	 * @param string $key Meta key & Option ID
	 * @param mixed $default Default value to return in case option is not set
	 * @return mixed
	 */
	function galactica_term_meta_option( $term_id, $prefix, $key, $default = false ) {

		if ( function_exists( 'get_term_meta' ) ) {
			$value = get_term_meta( $term_id, "{$prefix}_{$key}", true );
		}

		if ( !isset( $value ) || $value === '' ) { // NOTE: do not use empty because we have valid '0' values
			// IF there is no value fetch from options
			$value = galactica_option( $key, $default );
		}

		return $value;
	}
}

if ( !function_exists( 'galactica_flush_rewrite_rules_schedule' ) ) {
	/**
	 * Mark option to flush rewrite rules
	 *
	 * @return void
	 */
	function galactica_flush_rewrite_rules_schedule() {
		update_option( GALACTICA_THEME_SLUG . '_options_custom_slugs_changed', true, true );
	}
}

if ( !function_exists( 'galactica_flush_rewrite_rules_check' ) ) {
	/**
	 * Flush rewrite rules if needed
	 *
	 * @return void
	 */
	function galactica_flush_rewrite_rules_check() {
		$flush_rewrite_rules = get_option( GALACTICA_THEME_SLUG . '_options_custom_slugs_changed', false );
		if ( !empty( $flush_rewrite_rules ) ) {
			add_action( 'wp_loaded', 'flush_rewrite_rules' );
			delete_option( GALACTICA_THEME_SLUG . '_options_custom_slugs_changed' );
		}
	}
}

if ( !function_exists( 'galactica_recompile_options_css_schedule' ) ) {
	/**
	 * Mark option to re-compile options CSS
	 *
	 * @return void
	 */
	function galactica_recompile_options_css_schedule() {
		update_option( GALACTICA_THEME_SLUG . '_options_recompile_css', true, true );
	}
}

if ( !function_exists( 'galactica_recompile_options_css' ) ) {
	/**
	 * Re-compile options CSS
	 *
	 * @return void
	 */
	function galactica_recompile_options_css() {
		$recompile_css = get_option( GALACTICA_THEME_SLUG . '_options_recompile_css', false );
		if ( !empty( $recompile_css ) && class_exists( 'ReduxFrameworkInstances' ) && class_exists( 'ReduxFramework' ) ) {

			$redux = ReduxFrameworkInstances::get_instance( GALACTICA_THEME_OPTIONS );
			if ( $redux instanceof ReduxFramework ) {
				$redux->transients['last_save'] = time();
				$redux->transients['last_save_mode'] = 'normal';
				$redux->transients['changed_values'] = $redux->options;
				$redux->transients['run_compiler'] = 1;
				$redux->set_transients();

				delete_option( GALACTICA_THEME_SLUG . '_options_recompile_css' );
			}
		}
	}
}

if ( !function_exists( 'galactica_updater_check_updates' ) ) {
	/**
	 * Check for current theme updates and notify user in case there is an update or an error.
	 * NOTE: Apparently multisite installs require update from plugin
	 *
	 * @param string $envato_username Envato username
	 * @param string $envato_api_key Envato API key
	 * @param string $theme_author Theme author name
	 * @return void
	 */
	function galactica_updater_check_updates( $envato_username, $envato_api_key, $theme_author ) {

		// Envato library
		require_once( GALACTICA_FRAMEWORK_DIR . '/vendor/envato-wordpress-toolkit-library/envato-wp-theme-updater.php' );

		// Check for updates
		Envato_WP_Theme_Updater::init( $envato_username, $envato_api_key, $theme_author );

	}
}

if ( !function_exists( 'galactica_icon_fonts' ) ) {
	/**
	 * Returns all icon fonts available in a theme
	 *
	 * @return array
	 */
	function galactica_icon_fonts() {
		return isset( $GLOBALS['galactica_icon_fonts'] ) ? (array) $GLOBALS['galactica_icon_fonts'] : array();
	}
}

if ( !function_exists( 'galactica_icon_font_get_class' ) ) {
	/**
	 * Returns icon font class from font ID
	 *
	 * @param string $font_id
	 * @return string
	 */
	function galactica_icon_font_get_class( $font_id ) {

		$font_class = '';

		$default_icon_font = 'font-awesome';

		$fonts = galactica_icon_fonts();
		if ( !empty( $fonts ) ) {
			if ( !empty( $fonts[$font_id] ) ) {
				$font_class = $fonts[$font_id]['class'];
			} else if ( !empty( $fonts[$default_icon_font] ) ) {
				$font_class = $fonts[$default_icon_font]['class'];
			}
		}

		return $font_class;
	}
}

if ( !function_exists( 'galactica_is_woocommerce_activated' ) ) {
	/**
	 * Test if WooCommerce plugin is installed and activated
	 *
	 * @return bool
	 */
	function galactica_is_woocommerce_activated() {
		return class_exists( 'WooCommerce' );
	}
}

if ( !function_exists( 'galactica_is_wpml_activated' ) ) {
	/**
	 * Test if WPML plugin is installed and activated
	 *
	 * @return bool
	 */
	function galactica_is_wpml_activated() {
		return class_exists( 'SitePress' );
	}
}

if ( !function_exists( 'galactica_column_print_image' ) ) {
	/**
	 * Return featured image for custom column
	 *
	 * @param int $post_id The ID of the current post.
	 * @return void
	 */
	function galactica_column_print_image( $post_id ) {
		echo get_the_post_thumbnail( $post_id, array(50, 50) );
	}
}

if ( !function_exists( 'galactica_column_insert' ) ) {
	/**
	 * Insert columns
	 *
	 * @param array $columns An associative array with columns.
	 * @param array $insert_position_columns An associative array with new columns to be inserted in set key positions.
	 * @return array
	 */
	function galactica_column_insert( $columns, $insert_position_columns ) {

		$new_columns = array();
		foreach ( $columns as $key => $title ) {
			if ( isset( $insert_position_columns[$key] ) ) {
				// Insert columns into selected position
				$new_columns += $insert_position_columns[$key];

				unset( $insert_position_columns[$key] );
			}

			$new_columns[$key] = $title;
		}

		if ( !empty( $insert_position_columns ) ) {
			// Insert columns without found key at the end
			foreach ( $insert_position_columns as $insert_position_column ) {
				$new_columns += $insert_position_column;
			}
		}

		return $new_columns;
	}
}

if ( !function_exists( 'galactica_bool' ) ) {
	/**
	 * Convert user input to bool.
	 * Accepts yes/no, true/false and 1/0 combinations.
	 *
	 * @param string $bool
	 * @return bool
	 */
	function galactica_bool( $bool ) {
		return in_array( strtolower( $bool ), array( 'yes', 'true', '1' ) );
	}
}

if ( !function_exists( 'galactica_bool2string' ) ) {
	/**
	 * Convert bool to string.
	 * Returns boolean as string representation.
	 *
	 * @param bool $value
	 * @return string
	 */
	function galactica_bool2string( $value ) {
		return ( $value ? esc_html__( 'Yes', 'adama' ) : esc_html__( 'No', 'adama' ) );
	}
}

if ( !function_exists( 'galactica_int_range' ) ) {
	/**
	 * Force min and max range on integer number.
	 *
	 * @param int $number Input number
	 * @param int $min Minimum number allowed
	 * @param int $max Maximum number allowed
	 * @return int
	 */
	function galactica_int_range( $number, $min, $max = false ) {

		$number = intval( $number );

		if ( $min !== false && $number < $min ) {
			$number = $min;
		} else if ( $max !== false && $number > $max ) {
			$number = $max;
		}

		return $number;
	}
}

if ( !function_exists( 'galactica_phone_extract' ) ) {
	/**
	 * Removes all characters from phone number.
	 *
	 * @param string $phone
	 * @return string
	 */
	function galactica_phone_extract( $phone ) {

		// Remove everything but numbers
		$phone_href = preg_replace( '/[^0-9]/', '', $phone );

		$phone_number = '';
		if ( !empty( $phone_href ) ) {
			$phone_plus = '';
			if ( strpos( $phone, '+' ) === 0 ) {
				$phone_plus = '+';
			}
			$phone_number = $phone_plus . $phone_href;
		}

		return $phone_number;
	}
}

if ( !function_exists( 'galactica_in_array' ) ) {
	/**
	 * Check that data exists in array, if not return default value.
	 *
	 * @param mixed $data
	 * @param array $array
	 * @param mixed $default
	 * @return mixed
	 */
	function galactica_in_array( $data, $array, $default ) {
		if ( !in_array( $data, $array ) ) {
			$data = $default;
		}

		return $data;
	}
}

if ( !function_exists( 'galactica_lower_in_array' ) ) {
	/**
	 * Convert data value to lower case and check that data exists in array, if not return default value.
	 *
	 * @param mixed $data
	 * @param array $array
	 * @param mixed $default
	 * @return mixed
	 */
	function galactica_lower_in_array( $data, $array, $default ) {

		$data = strtolower( $data );

		if ( !in_array( $data, $array ) ) {
			$data = $default;
		}

		return $data;
	}
}


if ( !function_exists( 'galactica_esc_social_url' ) ) {
	/**
	 * Returns escaped URL for social network
	 *
	 * @param string $url
	 * @param string $slug
	 * @return string
	 */
	function galactica_esc_social_url( $url, $slug ) {
		if ( $slug === 'skype' && strpos( $url, 'skype:' ) !== 0 ) {
			$url = "skype:{$url}?call";
		}

		return esc_url( $url );
	}
}

if ( !function_exists( 'galactica_time_since' ) ) {
	/**
	 * Returns time since
	 *
	 * @param string $date
	 * @return string
	 */
	function galactica_time_since( $date ) {
		return human_time_diff( strtotime( $date ) );
	}
}

if ( !function_exists( 'galactica_file_mod_time' ) ) {
	/**
	 * Returns file modification time
	 *
	 * @param string $file_path
	 * @return string
	 */
	function galactica_file_mod_time( $file_path ) {
		return date( 'YmdHis', filemtime( $file_path ) );
	}
}

if ( !function_exists( 'galactica_shortcode_parse' ) ) {
	/**
	 * Parse shortcode attributes.
	 *
	 * @param string $content
	 * @return string
	 */
	function galactica_shortcode_parse( $content ) {
		// Match shortcodes
		$matches = array();
		$pattern = get_shortcode_regex();
		preg_match_all( "/$pattern/", $content, $matches );

		// Parse shortcode attributes
		$shortcodes = array();
		foreach ( $matches[2] as $key => $shortcode_name ) {
			if ( !isset( $shortcodes[$shortcode_name] ) ) {
				$shortcodes[$shortcode_name] = array();
			}

			$attributes = shortcode_parse_atts( $matches[3][$key] );
			$shortcodes[$shortcode_name][] = is_array( $attributes ) ? $attributes : array();
		}

		return $shortcodes;
	}
}

/**
 * Whether the current request is for a login/register page
 *
 * @return bool
 */
function galactica_is_login_register_page() {
	return in_array( $GLOBALS['pagenow'], array( 'wp-login.php', 'wp-register.php' ) );
}

if ( !function_exists( 'galactica_map' ) ) {
	/**
	 * Print HTML for Google map.
	 *
	 * @param array $atts
	 * @return string
	 */
	function galactica_map( $atts ) {
		if ( empty( $atts ) ) {
			return '';
		}

		// Enqueue Google Maps script
		wp_enqueue_script( 'google-maps-api' );

		$default_type = 'road';
		$atts = shortcode_atts( array(
			'type'                => $default_type,
			'address'             => '',
			'tooltip'             => '',
			'width'               => '100%',
			'height'              => '350px',
			'zoom'                => 15,
			'scrollwheel_enabled' => false,
		), $atts );

		$type = galactica_lower_in_array( $atts['type'], array( 'road', 'satellite', 'hybrid', 'terrain' ), $default_type );
		$zoom = galactica_int_range( $atts['zoom'], 1, 20 );

		$style = sprintf( 'width:%s%s;', $atts['width'], is_numeric( $atts['width'] ) ? 'px' : '' );
		$style .= sprintf( 'height:%s%s;', $atts['height'], is_numeric( $atts['height'] ) ? 'px' : '' );

		return sprintf(
			'<div class="map google-map" style="%s"
				data-type="%s"
				data-address="%s"
				data-tooltip="%s"
				data-zoom="%d"
				data-scrollwheel_enabled="%s">
			</div>',
			esc_attr( $style ),
			esc_attr( $type ),
			esc_attr( $atts['address'] ),
			esc_attr( $atts['tooltip'] ),
			$zoom,
			var_export( galactica_bool( $atts['scrollwheel_enabled'] ), true )
		);
	}
}

if ( !function_exists( 'galactica_animation_list' ) ) {
	/**
	 * Returns list of animation classes.
	 *
	 * @return array
	 */
	function galactica_animation_list() {

		$animations = array(
			'' => esc_html__( 'None', 'adama' ),
			esc_html__( 'Attention Seekers', 'adama' ) => array(
				'bounce'     => 'bounce',
				'flash'      => 'flash',
				'headShake'  => 'headShake',
				'jello'      => 'jello',
				'pulse'      => 'pulse',
				'rubberBand' => 'rubberBand',
				'shake'      => 'shake',
				'swing'      => 'swing',
				'tada'       => 'tada',
				'wobble'     => 'wobble',
			),
			esc_html__( 'Bouncing Entrances', 'adama' ) => array(
				'bounceIn'      => 'bounceIn',
				'bounceInDown'  => 'bounceInDown',
				'bounceInLeft'  => 'bounceInLeft',
				'bounceInRight' => 'bounceInRight',
				'bounceInUp'    => 'bounceInUp',
			),
			esc_html__( 'Bouncing Exits', 'adama' ) => array(
				'bounceOut'      => 'bounceOut',
				'bounceOutDown'  => 'bounceOutDown',
				'bounceOutLeft'  => 'bounceOutLeft',
				'bounceOutRight' => 'bounceOutRight',
				'bounceOutUp'    => 'bounceOutUp',
			),
			esc_html__( 'Fading Entrances', 'adama' ) => array(
				'fadeIn'      => 'fadeIn',
				'fadeInDown'  => 'fadeInDown',
				'fadeInLeft'  => 'fadeInLeft',
				'fadeInRight' => 'fadeInRight',
				'fadeInUp'    => 'fadeInUp',
			),
			esc_html__( 'Fading Exits', 'adama' ) => array(
				'fadeOut'      => 'fadeOut',
				'fadeOutDown'  => 'fadeOutDown',
				'fadeOutLeft'  => 'fadeOutLeft',
				'fadeOutRight' => 'fadeOutRight',
				'fadeOutUp'    => 'fadeOutUp',
			),
			esc_html__( 'Flippers', 'adama' ) => array(
				'flip'     => 'flip',
				'flipInX'  => 'flipInX',
				'flipInY'  => 'flipInY',
				'flipOutX' => 'flipOutX',
				'flipOutY' => 'flipOutY',
			),
			esc_html__( 'Lightspeed', 'adama' ) => array(
				'lightSpeedIn'  => 'lightSpeedIn',
				'lightSpeedOut' => 'lightSpeedOut',
			),
			esc_html__( 'Rotating Entrances', 'adama' ) => array(
				'rotateIn'          => 'rotateIn',
				'rotateInDownLeft'  => 'rotateInDownLeft',
				'rotateInDownRight' => 'rotateInDownRight',
				'rotateInUpLeft'    => 'rotateInUpLeft',
				'rotateInUpRight'   => 'rotateInUpRight',
			),
			esc_html__( 'Rotating Exits', 'adama' ) => array(
				'rotateOut'          => 'rotateOut',
				'rotateOutDownLeft'  => 'rotateOutDownLeft',
				'rotateOutDownRight' => 'rotateOutDownRight',
				'rotateOutUpLeft'    => 'rotateOutUpLeft',
				'rotateOutUpRight'   => 'rotateOutUpRight',
			),
			esc_html__( 'Sliding Entrances', 'adama' ) => array(
				'slideInUp'    => 'slideInUp',
				'slideInDown'  => 'slideInDown',
				'slideInLeft'  => 'slideInLeft',
				'slideInRight' => 'slideInRight',
			),
			esc_html__( 'Sliding Exits', 'adama' ) => array(
				'slideOutUp'    => 'slideOutUp',
				'slideOutDown'  => 'slideOutDown',
				'slideOutLeft'  => 'slideOutLeft',
				'slideOutRight' => 'slideOutRight',
			),
			esc_html__( 'Zoom Entrances', 'adama' ) => array(
				'zoomIn'      => 'zoomIn',
				'zoomInDown'  => 'zoomInDown',
				'zoomInLeft'  => 'zoomInLeft',
				'zoomInRight' => 'zoomInRight',
				'zoomInUp'    => 'zoomInUp',
			),
			esc_html__( 'Zoom Exits', 'adama' ) => array(
				'zoomOut'      => 'zoomOut',
				'zoomOutDown'  => 'zoomOutDown',
				'zoomOutLeft'  => 'zoomOutLeft',
				'zoomOutRight' => 'zoomOutRight',
				'zoomOutUp'    => 'zoomOutUp',
			),
			esc_html__( 'Specials', 'adama' ) => array(
				'hinge'   => 'hinge',
				'rollIn'  => 'rollIn',
				'rollOut' => 'rollOut',
			),
		);

		return $animations;
	}
}

if ( !function_exists( 'galactica_bootstrap_columns_class' ) ) {
	/**
	 * Returns Bootstrap column classes based on required column number.
	 *
	 * @param int $columns
	 * @return string
	 */
	function galactica_bootstrap_columns_class( $columns ) {

		$devices = array(
			1  => 'col-xs-12 col-sm-12 col-md-12 col-lg-12', // 1-1-1
			2  => 'col-xs-12 col-sm-6 col-md-6 col-lg-6',    // 1-2-2
			3  => 'col-xs-12 col-sm-6 col-md-4 col-lg-4',    // 1-2-3
			4  => 'col-xs-12 col-sm-6 col-md-3 col-lg-3',    // 1-2-4
			6  => 'col-xs-6 col-sm-3 col-md-2 col-lg-2',     // 2-4-6
			12 => 'col-xs-6 col-sm-2 col-md-1 col-lg-1',     // 2-6-12
		);

		$columns_class = '';
		if ( isset( $devices[$columns] ) ) {
			$columns_class = $devices[$columns];
		}

		return $columns_class;
	}
}

if ( !function_exists( 'galactica_theme_upload_file' ) ) {
	/**
	 * Returns file path and file url in theme's upload folder.
	 *
	 * @param string $filename
	 * @return array|bool
	 */
	function galactica_theme_upload_file( $filename ) {
		$upload_dir =  wp_upload_dir();

		$theme_slug = preg_replace( '/[^a-z0-9]/', '-', GALACTICA_THEME_SLUG );
		$theme_slug = preg_replace( '/\-+/', '-', $theme_slug );

		$theme_upload_dir = trailingslashit( $upload_dir['basedir'] ) . "galactica/{$theme_slug}/";

		if ( wp_mkdir_p( $theme_upload_dir ) ) {

			$theme_upload = array(
				'path' => "{$theme_upload_dir}{$filename}",
				'url'  => trailingslashit( $upload_dir['baseurl'] ) . "galactica/{$theme_slug}/{$filename}",
			);

			return $theme_upload;
		}

		return false;
	}
}

if ( !function_exists( 'galactica_get_query_var_paged' ) ) {
	/**
	 * Returns current paged/page query var or 1 if it's empty.
	 *
	 * @return integer
	 */
	function galactica_get_query_var_paged() {
		if ( !( $pg_var = intval( get_query_var( 'page' ) ) ) ) {
			$pg_var = intval( get_query_var( 'paged' ) );
			$pg_var = ( $pg_var > 0 ) ? $pg_var : 1;
		}

		return absint( $pg_var );
	}
}

if ( !function_exists( 'galactica_get_next_posts_url' ) ) {
	/**
	 * Returns next url
	 *
	 * @param int $max_page
	 * @return integer|boolean
	 */
	function galactica_get_next_posts_url( $max_page = 0 ) {
		global $paged, $wp_query;

		$paged = galactica_get_query_var_paged();

		if ( !$max_page ) {
			$max_page = $wp_query->max_num_pages;
		}

		$nextpage = intval( $paged ) + 1;

		if ( !is_single() && ( $nextpage <= $max_page ) ) {
			return next_posts( $max_page, false );
		}

		return false;
	}
}

if ( !function_exists( 'galactica_get_excerpt' ) ) {
	/**
	 * Custom excerpts return function
	 *
	 * @param int $length Excerpt length.
	 * @param bool $hard_limit Hard limit. If 'false' limit only excerpt generated from content
	 * @return string
	 */
	function galactica_get_excerpt( $length = 55, $hard_limit = false ) {
		// Sanitize data
		$limit = intval( $length );

		// If has custom excerpt
		if ( has_excerpt() ) {
			$excerpt = strip_shortcodes( get_the_excerpt() );

			// If excerpt is hard limited
			if ( $hard_limit ) {
				$excerpt = wp_trim_words( $excerpt, $limit );
			}
		} else {
			$excerpt = get_the_content();
			$excerpt = strip_shortcodes( $excerpt );
			$excerpt = apply_filters( 'the_content', $excerpt );
			$excerpt = str_replace( ']]>', ']]&gt;', $excerpt );
			$excerpt = wp_trim_words( $excerpt, $limit );
		}

		return apply_filters( 'get_the_excerpt', $excerpt );
	}
}

if ( !function_exists( 'galactica_excerpt' ) ) {
	/**
	 * Custom excerpts output function
	 *
	 * @param int $length Excerpt length.
	 * @param bool $hard_limit Hard limit. If 'false' limit only excerpt generated from content
	 * @return void
	 */
	function galactica_excerpt( $length = 55, $hard_limit = false ) {
		echo apply_filters( 'the_excerpt', galactica_get_excerpt( $length, $hard_limit ) );
	}
}

if ( !function_exists( 'galactica_format_quote_content' ) ) {
	/**
	 * Content for "quote" post format
	 *
	 * @return string
	 */
	function galactica_format_quote_content() {
		$before_content = '';
		$after_content = '';

		// If no <blockquote> elements were found, wrap the entire content in one
		if ( !preg_match( '/<blockquote.*?>/', get_the_content(), $matches ) ) {
			$before_content = '<blockquote>';
			$after_content = '</blockquote>';
		}

		echo $before_content;
			the_content();
		echo $after_content;
	}
}

if ( !function_exists( 'galactica_get_link_url' ) ) {
	/**
	 * Return the post URL for "link" post format
	 *
	 * Falls back to the post permalink if no URL is found in the post.
	 *
	 * @return string The Link format URL.
	 */
	function galactica_get_link_url() {
		$has_url = get_url_in_content( get_the_content() );

		return $has_url ? $has_url : apply_filters( 'the_permalink', get_permalink() );
	}
}

if( !function_exists( 'galactica_link_pages' ) ) {
	/**
	 * Displays page links for paginated posts
	 *
	 * @return void
	 */
	function galactica_link_pages() {
		wp_link_pages( array(
			'before' => '<div class="page-links"><div class="page-links-title">' . esc_html__( 'Pages:', 'adama' ) . '</div>',
			'after'  => '</div>'
		) );
	}
}

if ( !function_exists( 'galactica_get_page_id' ) ) {
	/**
	 * Fetch current page id for use in get_post_meta and similar functions
	 *
	 * @return int
	 */
	function galactica_get_page_id() {

		$page_id = null;

		// Blog
		if ( is_home() ) {
			$page_id = get_option( 'page_for_posts' );
		}

		// Product archive, we need to use WooCommerce Shop page
		if ( is_post_type_archive( 'product' ) ) {
			$page_id = wc_get_page_id( 'shop' );
		}

		// Single page
		if ( empty( $page_id ) && is_singular() ) {
			$page_id = get_queried_object()->ID;
		}

		return (int) $page_id;
	}
}

if ( !function_exists( 'galactica_get_attachment_image' ) ) {
	/**
	 * Get an HTML img element representing an image attachment
	 *
	 * @param int          $attachment_id Image attachment ID.
	 * @param string|array $html_attr     Optional. Attributes for the image markup. Default empty.
	 * @param array $attr                 Optional. Array of attributes. Default empty.
	 * @return string HTML img element or empty string on failure.
	 */
	function galactica_get_attachment_image( $attachment_id, $html_attr = '', $attr = array() ) {
		$html = '';
		$image = galactica_get_resized_image_src( $attachment_id, $attr );

		if ( $image ) {
			$src = $image['src'];
			$width = $image['width'];
			$height = $image['height'];
			$retina = !empty( $image['retina'] ) ? $image['retina'] : '';
			$hwstring = image_hwstring( $width, $height );

			$attachment = get_post( $attachment_id );
			$default_attr = array(
				'src'	=> $src,
				'alt'	=> trim( strip_tags( get_post_meta( $attachment_id, '_wp_attachment_image_alt', true ) ) ), // Use Alt field first
			);

			if ( empty( $default_attr['alt'] ) ) {
				$default_attr['alt'] = trim( strip_tags( $attachment->post_excerpt ) ); // If not, Use the Caption
			}
			if ( empty( $default_attr['alt'] ) ) {
				$default_attr['alt'] = trim( strip_tags( $attachment->post_title ) ); // Finally, use the title
			}

			$html_attr = wp_parse_args( $html_attr, $default_attr );

			// Generate 'srcset' if not already present.
			if ( !empty( $retina ) && empty( $attr['srcset'] ) ) {
				$html_attr['srcset'] = $src . ' 1x, ' . $retina['src'] . ' 2x';
			}

			/**
			 * Filter the list of attachment image attributes.
			 *
			 * @since 2.8.0
			 *
			 * @param array        $html_attr       Attributes for the image markup.
			 * @param WP_Post      $attachment Image attachment post.
			 */
			$html_attr = apply_filters( 'wp_get_attachment_image_attributes', $html_attr, $attachment );
			$html_attr = array_map( 'esc_attr', $html_attr );
			$html = rtrim( "<img $hwstring" );

			foreach ( $html_attr as $name => $value ) {
				$html .= " $name=" . '"' . $value . '"';
			}

			$html .= ' />';
		}

		return $html;
	}
}

if ( !function_exists( 'galactica_get_resized_image_src' ) ) {
	/**
	 * Returns resized image.
	 *
	 * @param int $img_id Image attachment ID.
	 * @param array $attr Optional. Array of attributes. Default empty.
	 * @return array
	 */
	function galactica_get_resized_image_src( $img_id, $attr = array() ) {
		if ( empty( $img_id ) ) {
			return false;
		}

		// Jetpack Photon support
		$photon_active = false;
		if ( class_exists( 'Jetpack' ) && method_exists( 'Jetpack', 'is_module_active' ) && Jetpack::is_module_active( 'photon' ) ) {
			$photon_active = remove_filter( 'image_downsize', array( Jetpack_Photon::instance(), 'filter_image_downsize' ) );
		}

		$original_img = wp_get_attachment_image_src( $img_id, 'full' );

		if ( $photon_active ) {
			add_filter( 'image_downsize', array( Jetpack_Photon::instance(), 'filter_image_downsize' ), 10, 3 );
		}

		if ( $original_img === false ) {
			// Image doesn't exist
			return false;
		}

		$thumb_width = 850;

		$old_w = $img_w = absint( $original_img[1] );
		$old_h = $img_h = absint( $original_img[2] );
		$img_src = $original_img[0];

		if ( $old_w === 0 || $old_h === 0 ) {
			return false;
		}

		$attr_defaults = array(
			'columns' => 1,
			'width'   => '',
			'height'  => '',
			'proportion'  => array(),
		);
		$attr = wp_parse_args( $attr, $attr_defaults );

		// Set default image data
		$return_data = array(
			'src'    => $img_src,
			'width'  => $old_w,
			'height' => $old_h,
		);

		// If not set width and height
		if ( empty( $attr['width'] ) && empty( $attr['height'] ) ) {
			$columns = absint( $attr['columns'] );

			if ( $columns === 1 ) {
				if ( is_page_template( 'templates/fullwidth.php' ) && ( galactica_option( 'layout', 'fluid' ) === 'fluid' ) ) {
					$img_w = 1920;
				} else {
					$img_w = 1140;

					if ( galactica_is_sidebar_enabled() ) {
						$img_w = $thumb_width;
					}
				}
			} else {
				$img_w = $thumb_width;
			}

			$ratio = $img_w / $old_w;
			$img_h = round( $old_h * $ratio, 0 );
		} else {
			$img_w = absint( $attr['width'] );
			$img_h = absint( $attr['height'] );

			// Empty height
			if ( empty( $attr['height'] ) ) {
				$ratio = $img_w / $old_w;
				$img_h = round( $old_h * $ratio, 0 );
			}
			// Empty width
			else if ( empty( $attr['width'] ) ) {
				$ratio = $img_h / $old_h;
				$img_w = round( $old_w * $ratio, 0 );
			}
		}

		if ( $old_w <= $img_w ) {
			$img_w = $old_w;
		}

		// Proportion
		if ( !empty( $attr['proportion'] ) ) {
			$proportion_w = empty( $attr['proportion'][0] ) ? 1 : $attr['proportion'][0];
			$proportion_h = empty( $attr['proportion'][1] ) ? 1 : $attr['proportion'][1];

			$img_h = round( $proportion_h * $img_w / $proportion_w );
		}

		if ( ( $img_w > $thumb_width ) && ( $img_w < $old_w ) || ( $img_h < $old_h ) ) {
			if ( $img_w > $old_w ) {
				$img_w = $old_w;
			}

			if ( $img_h > $old_h ) {
				$img_h = $old_h;
			}

			if ( !$photon_active ) {
				$file_arr = galactica_aq_resize( $img_src, $img_w, $img_h, true, false, false );

				$return_data['src'] = $file_arr[0];
				$return_data['width'] = $file_arr[1];
				$return_data['height'] = $file_arr[2];
			} else {
				$return_data['src'] = jetpack_photon_url( $img_src, array( 'resize' => array( $img_w, $img_h ) ) );
				$return_data['width'] = $img_w;
				$return_data['height'] = $img_h;
			}
		}

		// Retina size
		if ( galactica_is_retina_on() ) {
			$retina_w = $img_w * 2;
			$retina_h = $img_h * 2;

			if ( ( $retina_w < $old_w ) || ( $retina_h < $old_h ) ) {
				if ( $retina_w > $old_w ) {
					$retina_w = $old_w;
				}

				if ( $retina_h > $old_h ) {
					$retina_h = $old_h;
				}

				if ( !$photon_active ) {
					$retina_file_arr = galactica_aq_resize( $img_src, $retina_w, $retina_h, true, false, false );

					$return_data['retina']['src'] = $retina_file_arr[0];
					$return_data['retina']['width'] = $retina_file_arr[1];
					$return_data['retina']['height'] = $retina_file_arr[2];
				} else {
					$return_data['retina']['src'] = jetpack_photon_url( $img_src, array( 'resize' => array( $retina_w, $retina_h ) ) );
					$return_data['retina']['width'] = $retina_w;
					$return_data['retina']['height'] = $retina_h;
				}
			} else {
				$return_data['retina']['src'] = $img_src;
				$return_data['retina']['width'] = $old_w;
				$return_data['retina']['height'] = $old_h;
			}
		}

		return $return_data;
	}
}

/**
 * Retina on flag.
 *
 * @return boolean
 */
function galactica_is_retina_on() {
	return galactica_option( 'enable_retina', false );
}
