<?php
/**
 * Galactica Themes
 *
 * Functions used in template files.
 *
 * @package GalacticaThemes
 * @subpackage Theme
 */

// Don't allow direct access
if ( !defined( 'ABSPATH' ) ) {
	die;
}


if ( !function_exists( 'adama_layout_body_class' ) ) {
	/**
	 * Return body class that depends on page layout.
	 *
	 * @return string
	 */
	function adama_layout_body_class() {

		$body_class = '';

		// Override via custom fields
		// wp_reset_query();
		$page_id = galactica_get_page_id();
		if ( !empty( $page_id ) ) {
			$page_layout = get_post_meta( $page_id, 'adama_page_layout', true );
			if ( !empty( $page_layout ) ) {
				$body_class = "layout-{$page_layout}";
			}
		}

		if ( empty( $body_class ) ) {
			$layout = galactica_option( 'layout', 'fluid' );
			$body_class = "layout-{$layout}";
		}

		return $body_class;
	}
}

if ( !function_exists( 'adama_body_class' ) ) {
	/**
	 * Add specific CSS class to body
	 *
	 * @param array $classes List of css classes
	 * @return array
	 */
	function adama_body_class( $classes ) {

		// Page Layout
		$classes[] = adama_layout_body_class();

		// If product archive we need to add class for columns
		if ( is_post_type_archive( 'product' ) || is_tax( get_object_taxonomies( 'product' ) ) ) {
			$shop_columns = galactica_option( 'shop_columns', 3 );
			$classes[] = "columns-{$shop_columns}";
		}

		// Disable classic sidebar on fullwidth template
		if ( !is_page_template( 'templates/fullwidth.php' ) ) {
			$classes[] = 'template-normalwidth';
		}

		// Classic sidebar position
		$sidebar_body_class = galactica_sidebar_body_class();
		if ( !empty( $sidebar_body_class ) ) {
			$classes[] = $sidebar_body_class;
		}

		// Sticky Header
		$header_enable_sticky = galactica_option( 'header_enable_sticky', true );
		if ( $header_enable_sticky ) {
			$classes[] = 'sticky-header';
		}

		return $classes;
	}
}

/**
 * Output head title
 * NOTE: only in use on older WordPress version that do not support title-tag.
 *
 * @return void
 */
function adama_head_title() {

	?>
		<title><?php wp_title( '-', true, 'right' ); ?></title>
	<?php

}

if ( !function_exists( 'adama_page_title' ) ) {
	/**
	 * Output main page title
	 *
	 * @return void
	 */
	function adama_page_title() {
		$title = '';

		if ( is_front_page() && !is_singular( 'page' ) ) {

			// Homepage - if not a static page
			$title = get_bloginfo( 'description' );

		} else if ( is_home() ) {

			// Blog
			$title = get_the_title( get_option( 'page_for_posts' ) );

		} else if ( is_archive() ) {
			// Archives

			if ( is_post_type_archive() ) {
				// Post type archives

				$post_type = get_query_var( 'post_type' );

				if ( is_array( $post_type ) ) {
					$post_type = reset( $post_type );
				}

				if ( $post_type === 'product' && function_exists( 'wc_get_page_id' ) ) {
					$shop_id = wc_get_page_id( 'shop' );
					if ( !empty( $shop_id ) ) {
						$title = get_the_title( $shop_id );
					}
				}

				if ( empty( $title ) ) {
					$post_type = get_post_type_object( $post_type );
					if ( isset( $post_type->label ) && $post_type->label !== '' ) {
						$title = $post_type->label;
					} else {
						$title = $post_type->labels->menu_name;
					}
				}

			} else if ( is_day() ) {

				// Daily archive
				$title = sprintf( esc_html__( 'Daily Archives: %s', 'adama' ), get_the_date() );

			} else if ( is_month() ) {

				// Monthly archive
				$title = sprintf( esc_html__( 'Monthly Archives: %s', 'adama' ), get_the_date( esc_html_x( 'F Y', 'monthly archives date format', 'adama' ) ) );

			} else if ( is_year() ) {

				// Yearly archive
				$title = sprintf( esc_html__( 'Yearly Archives: %s', 'adama' ), get_the_date( esc_html_x( 'Y', 'yearly archives date format', 'adama' ) ) );

			} else if ( is_author() ) {

				// Author
				$title = sprintf( esc_html__( 'All Posts by: %s', 'adama' ), get_the_author() );

			} else {

				// Term title
				$title = single_term_title( '', false );

			}

		} else if ( is_search() ) {

			// Search
			$title = sprintf( esc_html__( 'Search Results for: &ldquo;%s&rdquo;', 'adama' ), get_search_query() );

		} else if ( is_404() ) {

			// 404
			$title = esc_html__( 'Page Not Found', 'adama' );

		} else {

			// Other
			$title = get_the_title();

		}

		// Allow child theme developers to override title tag
		$main_title_tag = apply_filters( 'adama/page/title/tag', 'h1' );

		printf( '<%1$s class="main-title">%2$s</%1$s>', $main_title_tag, $title );
	}
}

if ( !function_exists( 'adama_breadcrumb' ) ) {
	/**
	 * Output breadcrumbs
	 * Supports "WordPress SEO by Yoast" plugin
	 *
	 * @return void
	 */
	function adama_breadcrumb() {
		if ( !galactica_option( 'show_breadcrumb', true ) ) {
			return;
		}

		if ( function_exists( 'yoast_breadcrumb' ) ) {
			// Let WordPress SEO by Yoast plugin take over

			// Customize HTML
			add_filter( 'wpseo_breadcrumb_output_class', 'adama_wpseo_breadcrumb_output_class' );
			add_filter( 'wpseo_breadcrumb_output_wrapper', 'adama_wpseo_breadcrumb_output_wrapper' );

			// Output Breadcrumbs
			yoast_breadcrumb();
		} else {
			// Galactica Breadcrumbs
			galactica_breadcrumb_builder();
		}

	}
}

if ( !function_exists( 'adama_wpseo_breadcrumb_output_class' ) ) {
	/**
	 * Change the HTML class on the WP SEO breadcrumbs wrapper element.
	 *
	 * @return string
	 */
	function adama_wpseo_breadcrumb_output_class() {
		return 'breadcrumb hidden-xs';
	}
}

if ( !function_exists( 'adama_wpseo_breadcrumb_output_wrapper' ) ) {
	/**
	 * Change the html tag used by WP SEO breadcrumb.
	 *
	 * @return string
	 */
	function adama_wpseo_breadcrumb_output_wrapper() {
		return 'div';
	}
}

if ( !function_exists( 'adama_account_link' ) ) {
	/**
	 * Add MyAccount/LogOut link top top header menu
	 *
	 * @return string
	 */
	function adama_account_link() {

		$output = '';

		if ( galactica_option( 'header_top_nav_enable_login', true ) ) {

			if ( is_user_logged_in() ) {
				$output = sprintf( '<li class="account-link account-link-logout"><a href="%s">%s</a></li>', esc_url( wp_logout_url( home_url( '/' ) ) ), esc_html__( 'Log Out', 'adama' ) );
			} else {

				if ( galactica_is_woocommerce_activated() && function_exists( 'wc_get_page_id' ) ) {
					$myaccount_permalink = get_permalink( wc_get_page_id( 'myaccount' ) );
				} else {
					$myaccount_permalink = wp_login_url( get_permalink() );
				}

				$output = sprintf( '<li class="account-link account-link-login"><a href="%s">%s</a></li>', esc_url( $myaccount_permalink ), esc_html__( 'Log In', 'adama' ) );
			}
		}

		return $output;
	}
}

if ( !function_exists( 'adama_header_top_content' ) ) {
	/**
	 * Outputs top bar content based on user selection
	 *
	 * @param string $content_type Type of content to return.
	 * @param string $position Left or right position.
	 * @return string
	 */
	function adama_header_top_content( $content_type, $position ) {
		$output = '';

		switch ( $content_type ) {
			case 'contact_info':
				$has_content = false;

				$header_top_phone = galactica_option( 'header_top_phone', '' );
				if ( !empty( $header_top_phone ) ) {
					$has_content = true;

					// Remove everything but numbers
					$phone_number = galactica_phone_extract( $header_top_phone );

					$phone_href = '';
					if ( !empty( $phone_number ) ) {
						$phone_href = sprintf( ' href="tel:%s"', esc_attr( $phone_number ) );
					}

					$header_top_phone = sprintf( '<span class="top-contact-info"><i class="fa fa-phone"></i><a%s>%s</a></span>', $phone_href, esc_html( $header_top_phone ) );
				}

				$header_top_email = galactica_option( 'header_top_email', '' );
				if ( !empty( $header_top_email ) ) {
					$has_content = true;
					$header_top_email = sprintf( '<span class="top-contact-info"><i class="fa fa-envelope"></i><a href="mailto:%s">%s</a></span>', esc_attr( $header_top_email ), esc_html( $header_top_email ) );
				}

				$header_top_address = galactica_option( 'header_top_address', '' );
				if ( !empty( $header_top_address ) ) {
					$has_content = true;
					$header_top_address = sprintf( '<span class="top-contact-info"><i class="fa fa-map-marker"></i>%s</span>', esc_html( $header_top_address ) );
				}

				if ( $has_content ) {
					$output = sprintf( '<span class="aux-text">%s%s%s</span>',
						$header_top_phone,
						$header_top_email,
						$header_top_address
					);
				}
				break;

			case 'secondary_menu':
				$menu_items = wp_nav_menu( array(
					'theme_location' => 'secondary',
					'container'      => false,
					'items_wrap'     => '%3$s',
					'sort_column'    => 'menu_order',
					'depth'          => 2,
					'fallback_cb'    => false,
					'echo'           => false,
				) );

				$menu_items .= adama_account_link();

				$menu_items .= adama_wpml_language_popup();

				if ( function_exists( 'adama_wc_cart_popup' ) ) {
					$menu_items .= adama_wc_cart_popup();
				}

				if ( !empty( $menu_items ) ) {
					$output = sprintf( '<nav class="top-header-menu"><ul class="menu">%s</ul></nav>', $menu_items );
				}
				break;

			case 'social_links':
				$social_list = galactica_social_list( 'top-header' );

				$nofollow = galactica_option( 'social_nofollow', true ) ? ' rel="nofollow"' : '';
				$target = esc_attr( galactica_option( 'social_target', '_blank' ) );

				$output = '';
				foreach ( $social_list as $social ) {

					$social_url = galactica_option( "social_{$social['slug']}", '' );

					if ( !empty( $social_url ) ) {
						$output .= sprintf(
							'<li class="social-%s">
								<a href="%s" target="%s"%s><i class="fa %s"></i></a>
							</li>',
							esc_attr( $social['slug'] ),
							galactica_esc_social_url( $social_url, $social['slug'] ),
							$target,
							$nofollow,
							esc_attr( $social['icon'] )
						);
					}
				}

				if ( !empty( $output ) ) {
					$output = '<ul class="social-icons">' . $output . '</ul>';
				}
				break;

			case 'text':
				$header_top_text = galactica_option( 'header_top_text', '' );
				if ( !empty( $header_top_text ) ) {
					$output = sprintf( '<span class="aux-text">%s</span>', $header_top_text );
				}
				break;
		}

		return $output;
	}
}

if ( !function_exists( 'adama_header_hero' ) ) {
	/**
	 * Outputs hero image or slider
	 *
	 * @return string
	 */
	function adama_header_hero() {

		$page_id = galactica_get_page_id();

		// Skip on archive pages
		if ( empty( $page_id ) ) {
			return '';
		}

		$hero_type = get_post_meta( $page_id, 'adama_hero_type', true );
		if ( empty( $hero_type ) ) {
			return '';
		}

		$output = '';
		$template_wrap_begin = '<div class="full-width full-width-hero">';
		$template_wrap_end = '</div>';

		switch ( $hero_type ) {
			case 'image':
				$image_id = get_post_meta( $page_id, 'adama_hero_image_id', true );

				if ( !empty( $image_id ) ) {
					$image_html = wp_get_attachment_image( $image_id, 'full' );

					if ( !empty( $image_html ) ) {
						$output = sprintf(
							'<div class="hero-image">%1$s</div>',
							$image_html
						);
					}
				}
				break;

			case 'content':
				$title_desc = '';
				$image_id = get_post_meta( $page_id, 'adama_hero_background_image_id', true );
				$title = get_post_meta( $page_id, 'adama_hero_title', true );
				$descripion = get_post_meta( $page_id, 'adama_hero_description', true );
				$style = '';

				if ( !empty( $image_id ) ) {
					$image = wp_get_attachment_image_src( $image_id, 'full' );

					if ( !empty( $image[0] ) ) {
						$style = sprintf(
							'background-image: url(%1$s);',
							esc_url( $image[0] )
						);
					}
				}

				if ( !empty( $title ) || !empty( $descripion ) ) {
					$color = get_post_meta( $page_id, 'adama_hero_color', true );
					$bg_color = get_post_meta( $page_id, 'adama_hero_bg_color', true );
					$align = get_post_meta( $page_id, 'adama_hero_align', true );

					if ( !empty( $title ) ) {
						$title = '<h2 class="page-header-title">' . $title . '</h2>';
					}

					if ( !empty( $descripion ) ) {
						$descripion = '<p>' . $descripion . '</p>';
					}

					if ( !empty( $color ) ) {
						$style .= 'color: ' . esc_attr( $color ) . ';';
					}

					if ( !empty( $bg_color ) ) {
						$style .= 'background-color: ' . esc_attr( $bg_color ) . ';';
					}

					if ( !empty( $align ) ) {
						$style .= 'text-align: ' . esc_attr( $align ) . ';';
					}

					$title_desc = sprintf(
						'<section class="page-header">
							<div class="container">
								%1$s%2$s
							</div>
						</section>',
						$title,
						$descripion
					);
				}

				if ( !empty( $style ) ) {
					$style = ' style="' . $style . '"';
				}

				$output = sprintf(
					'<div class="hero-type-content"%1$s>
						%2$s
					</div>',
					$style,
					$title_desc
				);
				break;

			case 'slider':
				if ( function_exists( 'galactica_core_slider_html' ) ) {
					$slider_type = get_post_meta( $page_id, 'adama_hero_slider_type', true );
					$slider_id = get_post_meta( $page_id, 'adama_hero_slider_id', true );

					if ( !empty( $slider_type ) && !empty( $slider_id ) ) {
						$atts = array(
							'type'       => $slider_type,
							'id'         => $slider_id,
							'full_width' => false,
						);

						if ( $slider_type === 'simple' ) {
							$atts += array(
								'order_by'        => galactica_term_meta_option( $slider_id, 'galactica', 'simple_slider_order_by', true ),
								'order'           => galactica_term_meta_option( $slider_id, 'galactica', 'simple_slider_order', true ),
								'loop'            => galactica_term_meta_option( $slider_id, 'galactica', 'simple_slider_loop', true ),
								'show_nav'        => galactica_term_meta_option( $slider_id, 'galactica', 'simple_slider_show_nav', false ),
								'show_indicators' => galactica_term_meta_option( $slider_id, 'galactica', 'simple_slider_show_indicators', true ),
								'autoplay'        => galactica_term_meta_option( $slider_id, 'galactica', 'simple_slider_autplay', true ),
								'interval'        => galactica_term_meta_option( $slider_id, 'galactica', 'simple_slider_interval', 3000 ),
								'hide_title'      => galactica_term_meta_option( $slider_id, 'galactica', 'simple_slider_hide_title', false ),
							);
						}

						$output = galactica_core_slider_html( $atts );
					}
				}
				break;

//			case 'video':
//				break;

		}

		return $template_wrap_begin . $output . $template_wrap_end;
	}
}

if ( !function_exists( 'adama_portfolio_slider' ) ) {
	/**
	 * Outputs portfolio slider
	 *
	 * @return string
	 */
	function adama_portfolio_slider() {

		if ( !function_exists( 'galactica_core_simple_slider_wrap_html' ) ) {
			return '';
		}

		$output = '';

		$gallery = get_post_meta( get_the_ID(), 'adama_portfolio_gallery', true );
		if ( !empty( $gallery ) ) {
			foreach ( $gallery as $image_id => $image_src ) {

				$output .= sprintf(
					'<div class="carousel-item">%s</div>',
					galactica_get_attachment_image( $image_id )
				);

			}

			if ( !empty( $output ) ) {
				$carousel_atts = array(
					'items'           => 1,
					'loop'            => true,
					'interval'        => 4000,
					'show_nav'        => true,
					'show_indicators' => false,
				);
				$output = galactica_core_simple_slider_wrap_html( $carousel_atts, $output, 'slider' );
			}

			if ( !empty( $output ) ) {
				$output = '<div class="portfolio-slider">' . $output . '</div>';
			}
		}

		return $output;
	}
}

if ( !function_exists( 'adama_get_single_portfolio_layout_class' ) ) {
	/**
	 * Returns layout class for single portfolio page
	 *
	 * @return string
	 */
	function adama_get_single_portfolio_layout_class() {
		$portfolio_class = 'portfolio-' . galactica_option( 'portfolio_single_layout', 'vertical' );
		$override_layout = get_post_meta( get_the_ID(), 'adama_portfolio_single_layout', true );

		if ( !empty( $override_layout ) ) {
			$portfolio_class = 'portfolio-' . $override_layout;
		}

		return $portfolio_class;
	}
}

if ( !function_exists( 'adama_post_get_related' ) ) {
	/**
	 * Outputs related posts
	 *
	 * @param int $post_id Post ID. Optional
	 * @return array
	 */
	function adama_post_get_related( $post_id = null ) {

		if ( !function_exists( 'galactica_core_html_post' ) ) {
			return '';
		}

		$post_id = ( null === $post_id ) ? get_the_ID() : (int) $post_id;
		$query_args = array();
		$html = '';

		switch ( get_post_meta( $post_id, 'adama_post_related_posts_source', true ) ) {
			case 'categories':
				$cats = wp_get_object_terms( $post_id, 'category', array( 'fields' => 'ids' ) );

				if ( $cats && !is_wp_error( $cats ) ) {
					$query_args['category__in'] = $cats;
					$query_args['post__not_in'] = array( $post_id );
				}

				break;
			case 'custom':
				$related_posts = get_post_meta( $post_id, 'adama_post_related_posts', true );

				if ( !empty( $related_posts ) ) {
					$query_args['post__in'] = $related_posts;
					$query_args['order'] = 'ASC';
					$query_args['orderby'] = 'post__in';
				}

				break;
		}

		if ( !empty( $query_args ) ) {
			$query_args['posts_per_page'] = 3;

			$html = '<div class="related-posts-container"><h3 class="section-title">' . esc_html__( 'Related Posts', 'adama' ) . '</h3>';
				$html .= galactica_core_html_post( array( 'layout' => 'hover', 'include_without_thumb' => true, 'columns' => 3 ), $query_args );
			$html .= '</div>';
		}

		return $html;
	}
}

if ( !function_exists( 'adama_post_slider' ) ) {
	/**
	 * Outputs post slider
	 *
	 * @return string
	 */
	function adama_post_slider() {

		if ( !function_exists( 'galactica_core_simple_slider_wrap_html' ) ) {
			return '';
		}

		$output = '';

		$gallery = get_post_meta( get_the_ID(), 'adama_post_gallery', true );
		if ( !empty( $gallery ) ) {
			foreach ( $gallery as $image_id => $image_src ) {

				$output .= sprintf(
					'<div class="carousel-item">%s</div>',
					galactica_get_attachment_image( $image_id )
				);

			}

			if ( !empty( $output ) ) {
				$carousel_atts = array(
					'items'           => 1,
					'loop'            => true,
					'interval'        => 4000,
					'show_nav'        => true,
					'show_indicators' => false,
				);
				$output = galactica_core_simple_slider_wrap_html( $carousel_atts, $output, 'slider' );
			}

			if ( !empty( $output ) ) {
				$output = '<div class="post-slider">' . $output . '</div>';
			}
		}

		return $output;
	}
}

if ( !function_exists( 'adama_wpml_language_current' ) ) {
	/**
	 * Outputs current language item
	 *
	 * @param string $language Current language.
	 * @param array $language_settings WPML settings.
	 * @return string
	 */
	function adama_wpml_language_current( $language, $language_settings ) {

		$output = '';
		$is_flag_shown = false;

		// Show flag if selected in WPML options
		if ( $language_settings['icl_lso_flags'] && $language['country_flag_url'] ) {
			$output .= sprintf( '<img src="%s" height="12" width="18" class="language-flag" alt="%s" title="%s" />',
				esc_url( $language['country_flag_url'] ),
				esc_attr( $language['language_code'] ),
				esc_attr( $language_settings['icl_lso_display_lang'] ? $language['translated_name'] : $language['native_name'] )
			);

			$is_flag_shown = true;
		}

		// Show name if selected in WPML options or if flag is not visible (making sure at least one is visible)
		if ( !$is_flag_shown || $language_settings['icl_lso_native_lang'] || $language_settings['icl_lso_display_lang'] ) {
			$output .= sprintf( '<span class="language name">%s</span>', ICL_LANGUAGE_NAME );
		}

		return $output;
	}
}

if ( !function_exists( 'adama_wpml_language_popup' ) ) {
	/**
	 * Outputs language popup menu
	 *
	 * @return string
	 */
	function adama_wpml_language_popup() {
		$output = '';

		if ( galactica_is_wpml_activated() ) {

			$default_settings = array(
				'icl_lso_link_empty'   => true,
				'icl_lso_flags'        => false,
				'icl_lso_native_lang'  => true,
				'icl_lso_display_lang' => false,
			);
			$language_settings = wp_parse_args( (array) get_option( 'icl_sitepress_settings' ), $default_settings );

			$languages = icl_get_languages( 'skip_missing=' . !$language_settings['icl_lso_link_empty'] . '&orderby=custom' );
			if ( !empty( $languages ) ) {
				ob_start();
				?>

				<li class="aux-languages dropdown">
					<a href="#"><?php echo adama_wpml_language_current( $languages[ICL_LANGUAGE_CODE], $language_settings ); ?></a>

					<ul id="auxLanguages" class="sub-menu animate-wr animated fadeInLeft">

						<?php foreach ( $languages as $language ) : ?>
							<li class="<?php echo !empty( $language['active'] ) ? 'current-menu-item active' : ''; ?>">
								<a href="<?php echo esc_url( $language['url'] ); ?>">
									<?php // Language name
										$lang_name = '';
										if ( $language_settings['icl_lso_native_lang'] && $language_settings['icl_lso_display_lang'] ) {
											$lang_name = "{$language['native_name']} ({$language['translated_name']})";
										} else if ( $language_settings['icl_lso_native_lang'] ) {
											$lang_name = $language['native_name'];
										} else if ( $language_settings['icl_lso_display_lang'] ) {
											$lang_name = $language['translated_name'];
										} ?>

									<div class="flag-language-wrap<?php echo !empty( $lang_name ) ? ' switcher-with-text' : ''; ?>">
										<?php if ( $language_settings['icl_lso_flags'] && $language['country_flag_url'] ) : ?>
											<img src="<?php echo esc_url( $language['country_flag_url'] ); ?>" height="12" width="18" class="language-flag" alt="<?php echo esc_attr( $language['language_code'] ); ?>"  title="<?php echo esc_attr( $language_settings['icl_lso_display_lang'] ? $language['translated_name'] : $language['native_name'] ); ?>" />
										<?php endif; ?>

										<?php if ( $lang_name ) : ?>
											<span class="language <?php echo $language['active'] ? 'language-active' : ''; ?>"><?php echo $lang_name; ?></span>
										<?php endif; ?>
									</div>
								</a>
							</li>
						<?php endforeach; ?>

					</ul>
				</li>

				<?php
				$output = ob_get_clean();
			}
		}

		return $output;
	}
}

if ( !function_exists( 'adama_footer_widgets' ) ) {
	/**
	 * Outputs footer widget area, if enabled
	 *
	 * @return void
	 */
	function adama_footer_widgets() {

		$footer_widget_count = intval( galactica_option( 'footer_widget_areas', 4 ) );
		if ( $footer_widget_count ) {
			$widget_col_class = 'footer-col-' . $footer_widget_count;
			?>
				<div class="footer-top">
					<div class="container">
						<div class="row">
							<?php for ( $i = 1; $i <= $footer_widget_count; $i++ ) : ?>

								<div class="col footer-col <?php echo $widget_col_class; ?>">
									<?php dynamic_sidebar( sprintf( 'galactica-footer-%d', $i ) ); ?>
								</div>

							<?php endfor; ?>
						</div>
					</div>
				</div>
			<?php
		}
	}
}

if ( !function_exists( 'adama_footer_left' ) ) {
	/**
	 * Left footer content - copyright text.
	 *
	 * @return void
	 */
	function adama_footer_left() {
		// Show copyright text
		echo galactica_mini_shortcode( galactica_option( 'footer_copyright_text', '' ) );

		// Show footer menu
		wp_nav_menu( array(
			'theme_location' => 'footer',
			'container'      => false,
			'depth'          => -1,
			'fallback_cb'    => false,
		) );
	}
}

if ( !function_exists( 'adama_footer_right' ) ) {
	/**
	 * Right footer content - credits text or social icons.
	 *
	 * @return void
	 */
	function adama_footer_right() {
		$enable_social = galactica_option( 'footer_enable_social', true );
		if ( $enable_social ) {
			$social_list = galactica_social_list( 'footer' );

			$nofollow = galactica_option( 'social_nofollow', true ) ? ' rel="nofollow"' : '';
			$target = esc_attr( galactica_option( 'social_target', '_blank' ) );

			$output = '';
			foreach ( $social_list as $social ) {

				$social_url = galactica_option( "social_{$social['slug']}", '' );

				if ( !empty( $social_url ) ) {
					$output .= sprintf(
						'<li class="social-%s">
							<a href="%s" target="%s"%s><i class="fa %s"></i></a>
						</li>',
						esc_attr( $social['slug'] ),
						galactica_esc_social_url( $social_url, $social['slug'] ),
						$target,
						$nofollow,
						esc_attr( $social['icon'] )
					);
				}
			}

			if ( !empty( $output ) ) {
				echo '<ul class="social-icons">' . $output . '</ul>';
			}
		} else {
			echo galactica_mini_shortcode( galactica_option( 'footer_credits_text', '' ) );
		}
	}
}

if ( !function_exists( 'adama_form_error' ) ) {
	/**
	 * Used to show contact form error.
	 *
	 * @param array $errors Error messages.
	 * @param string $name Control name.
	 * @return void
	 */
	function adama_form_error( $errors, $name ) {
		if ( !empty( $errors[$name] ) ) {
			printf( '<div class="alert alert-danger alert-form-message">%s</div>', $errors[$name] );
		}
	}
}

if ( !function_exists( 'adama_get_blog_layout' ) ) {
	/**
	 * Returns blog layout
	 *
	 * @return string
	 */
	function adama_get_blog_layout() {
		return galactica_option( 'blog_layout', 'list' );
	}
}

if ( !function_exists( 'adama_get_blog_layout_class' ) ) {
	/**
	 * Returns layout class for blog
	 *
	 * @return string
	 */
	function adama_get_blog_layout_class() {
		$blog_layout = adama_get_blog_layout();
		$class = "blog-{$blog_layout}";

		if ( $blog_layout === 'masonry' ) {
			$class .= ' masonry';
		}

		return $class;
	}
}

if ( !function_exists( 'adama_get_blog_columns_class' ) ) {
	/**
	 * Returns column class for blog
	 *
	 * @return string
	 */
	function adama_get_blog_columns_class() {
		$blog_columns = '';

		if ( adama_get_blog_layout() === 'masonry' ) {
			$blog_columns = 'masonry-column-' . galactica_option( 'blog_masonry_columns', '4' );
		}

		return $blog_columns;
	}
}

if ( !function_exists( 'adama_social_share' ) ) {
	/**
	 * Single product, Social share options
	 *
	 * @return void
	 */
	function adama_social_share() {
		$social_options = array(
			'facebook' => array(
				'url'  => 'https://www.facebook.com/sharer/sharer.php?u=%1$s&amp;t=%2$s',
				'text' => esc_html__( 'Share on Facebook', 'adama' ),
			),
			'twitter' => array(
				'url'  => 'https://www.twitter.com/share?url=%1$s&text=%2$s',
				'text' => esc_html__( 'Share on Twitter', 'adama' ),
			),
			'pinterest-p' => array(
				'url'  => 'https://www.pinterest.com/pin/create/button/?url=%1$s&description=%2$s&media=%3$s',
				'text' => esc_html__( 'Share on Pinterest', 'adama' ),
			),
			'envelope' => array(
				'url'  => 'mailto:?subject=%2$s&body=%1$s',
				'text' => esc_html__( 'Share via Email', 'adama' ),
			),
		);

		$nofollow = apply_filters( 'adama/shop/share/nofollow', true ) ? ' rel="nofollow"' : '';
		$social_options = apply_filters( 'adama/shop/share/social_options', $social_options );

		$output = '';
		foreach ( $social_options as $name => $options ) {
			$full_image = wp_get_attachment_image_src( get_post_thumbnail_id(), 'full' );
			$url = sprintf( $options['url'], urlencode( get_permalink() ), urlencode( get_the_title() ), urlencode( $full_image[0] ) );

			$output .= sprintf(
				'<li class="%1$s">
					<a href="%2$s" target="_blank"%3$s><i class="fa fa-%1$s"></i><span>%4$s</span></a>
				</li>',
				$name,
				esc_url( $url ),
				$nofollow,
				esc_html( $options['text'] )
			);
		}

		echo '<ul class="social-share">', $output, '</ul>';
	}
}
