<?php
/**
 * Galactica Themes
 *
 * WooCommerce related functions and hooks
 *
 * @package GalacticaThemes
 * @subpackage WooCommerce
 */

// Don't allow direct access
if ( !defined( 'ABSPATH' ) ) {
	die;
}


// Cookie and query string name for number of items per page
define( 'GALACTICA_WC_SHOP_ITEMS_PER_PAGE', 'shop-products-per-page' );

// Remove default WooCommerce styles
add_filter( 'woocommerce_enqueue_styles', '__return_false' );

// Remove WooCommerce breadcrumb
remove_action( 'woocommerce_before_main_content', 'woocommerce_breadcrumb', 20, 0 );

if ( !function_exists( 'adama_wc_remove_scripts' ) ) {
	/**
	 * Remove scripts initialized by WooCommerce
	 *
	 * @return void
	 */
	function adama_wc_remove_scripts() {

		// Remove PrettyPhoto CSS/JS
		wp_dequeue_style( 'woocommerce_prettyPhoto_css' );
		wp_dequeue_script( 'prettyPhoto' );
		wp_dequeue_script( 'prettyPhoto-init' );

	}
}
add_action( 'wp_enqueue_scripts', 'adama_wc_remove_scripts', 100 );

if ( !function_exists( 'adama_wc_cart_popup' ) ) {
	/**
	 * WooCommerce popup cart
	 *
	 * @return string
	 */
	function adama_wc_cart_popup() {
		$output = '';

		if ( galactica_is_woocommerce_activated() && galactica_option( 'header_top_nav_enable_cart', true ) ) {
			ob_start();
			get_template_part( 'templates/cart-popup' );
			$output = ob_get_clean();
		}

		return $output;
	}
}

if ( !function_exists( 'adama_wc_widgets_init' ) ) {
	/**
	 * WooCommerce init shop sidebar
	 *
	 * @return void
	 */
	function adama_wc_widgets_init() {

		if ( galactica_is_woocommerce_activated() ) {
			// Shop sidebar
			register_sidebar( array(
				'name'          => esc_html__( 'Shop', 'adama' ),
				'id'            => 'galactica-shop',
				'description'   => esc_html__( "Widgets in this area will be shown in website's sidebar on shop pages.", 'adama' ),
				'before_widget' => '<div id="%1$s" class="widget %2$s">',
				'after_widget'  => '</div>',
				'before_title'  => '<h3 class="widgettitle">',
				'after_title'   => '</h3>',
			) );
		}

	}
}
add_filter( 'widgets_init', 'adama_wc_widgets_init' );

if ( !function_exists( 'adama_add_to_cart_fragments' ) ) {
	/**
	 * WooCommerce popup cart ajax fragment
	 *
	 * @param array $fragments WooCommerce fragments.
	 * @return array
	 */
	function adama_add_to_cart_fragments( $fragments ) {
		if ( galactica_option( 'header_top_nav_enable_cart', true ) ) {
			$fragments['.shopping-cart'] = adama_wc_cart_popup();
		}

		return $fragments;
	}
}
// NOTE: Since WooCommerce v2.3 use woocommerce_add_to_cart_fragments istead of add_to_cart_fragments
add_filter( 'woocommerce_add_to_cart_fragments', 'adama_add_to_cart_fragments' );

if ( !function_exists( 'adama_woocommerce_show_page_title' ) ) {
	/**
	 * WooCommerce hide title
	 *
	 * @return bool
	 */
	function adama_woocommerce_show_page_title() {
		//if ( is_shop() && is_singular( 'product' ) ) {
			return false;
		//}
	}
}
add_filter( 'woocommerce_show_page_title', 'adama_woocommerce_show_page_title' );

if ( !function_exists( 'adama_loop_shop_per_page_set' ) ) {
	/**
	 * Product archive/category pages - Change number of products shown
	 *
	 * @return void
	 */
	function adama_loop_shop_per_page_set() {
		// Did user select new value
		if ( isset( $_REQUEST[GALACTICA_WC_SHOP_ITEMS_PER_PAGE] ) ) {
			$posts_per_page = intval( $_REQUEST[GALACTICA_WC_SHOP_ITEMS_PER_PAGE] );

			// Save to session and cookie
			if ( !empty( $posts_per_page ) ) {
				$_SESSION[GALACTICA_WC_SHOP_ITEMS_PER_PAGE] = $posts_per_page;
				setcookie( GALACTICA_WC_SHOP_ITEMS_PER_PAGE, $posts_per_page, time() + 1209600, COOKIEPATH, COOKIE_DOMAIN, false );
			}
		}
	}
}
add_filter( 'init', 'adama_loop_shop_per_page_set', 50 );

if ( !function_exists( 'adama_loop_shop_per_page' ) ) {
	/**
	 * Product archive/category pages - Change number of products shown
	 *
	 * @return int
	 */
	function adama_loop_shop_per_page() {
		// Try to read from session
		if ( isset( $_SESSION[GALACTICA_WC_SHOP_ITEMS_PER_PAGE] ) ) {
			$posts_per_page = intval( $_SESSION[GALACTICA_WC_SHOP_ITEMS_PER_PAGE] );
		}

		// Try to read from cookie
		if ( empty( $posts_per_page ) && isset( $_COOKIE[GALACTICA_WC_SHOP_ITEMS_PER_PAGE] ) ) {
			$posts_per_page = intval( $_COOKIE[GALACTICA_WC_SHOP_ITEMS_PER_PAGE] );
		}

		// Default
		if ( empty( $posts_per_page ) ) {
			$posts_per_page = 12;
		}

		return $posts_per_page;
	}
}
add_filter( 'loop_shop_per_page', 'adama_loop_shop_per_page', 50 );

if ( !function_exists( 'adama_add_to_cart_spinner' ) ) {
	/**
	 * Product archive/category pages - Add spinner
	 *
	 * @return void
	 */
	function adama_add_to_cart_spinner() {
		echo '<div class="add-to-cart-spinner"><i class="fa fa-spinner fa-spin"></i></div>';
	}
}
add_action( 'woocommerce_before_shop_loop_item_title', 'adama_add_to_cart_spinner', 20 );

if ( !function_exists( 'adama_woocommerce_shop_columns' ) ) {
	/**
	 * Change columns number for the shop.
	 *
	 * @return void
	 */
	function adama_woocommerce_shop_columns() {
		return galactica_option( 'shop_columns', 3 );
	}
}
add_action( 'loop_shop_columns', 'adama_woocommerce_shop_columns' );

remove_action( 'woocommerce_after_shop_loop', 'woocommerce_pagination', 10 );
add_action( 'woocommerce_after_shop_loop', 'adama_pagination', 10 );

// Remove Add to cart button
remove_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_add_to_cart', 10 );

// Remove anchor tag for products in the loop
remove_action( 'woocommerce_before_shop_loop_item', 'woocommerce_template_loop_product_link_open', 10 );
remove_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_product_link_close', 5 );

// Single product - Switch rating position
remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_rating', 10 );
add_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_rating', 11 );

// Single product - show social share buttons
if ( galactica_option( 'shop_enable_share', true ) ) {
	add_action( 'woocommerce_after_single_product_summary', 'adama_social_share', 12 );
}

/**
 * Single product, Upsell products - show 4 products in 4 columns
 *
 * @return void
 */
function adama_woocommerce_after_single_product_summary() {
	woocommerce_upsell_display( 4, 4 );
}
remove_action( 'woocommerce_after_single_product_summary', 'woocommerce_upsell_display', 15 );
add_action( 'woocommerce_after_single_product_summary', 'adama_woocommerce_after_single_product_summary', 15 );

/**
 * Single product, Related products - show 4 products in 4 columns
 *
 * @param array $args Arguments to customize related products output.
 * @return array
 */
function adama_woocommerce_output_related_products_args( $args ) {

	if ( is_singular( 'product' ) ) {
		$args['posts_per_page'] = 4;
		$args['columns'] = 4;
	}

	return $args;
}
add_filter( 'woocommerce_output_related_products_args', 'adama_woocommerce_output_related_products_args' );

/**
 * Cart, Cross-sell products - show 4 products in 4 columns
 *
 * @return void
 */
function adama_woocommerce_cross_sell_display() {
	woocommerce_cross_sell_display( 2, 2 );
}
remove_action( 'woocommerce_cart_collaterals', 'woocommerce_cross_sell_display' );
add_action( 'woocommerce_cart_collaterals', 'adama_woocommerce_cross_sell_display' );

/**
 * Returns gravatar size for WooCommerce reviews.
 *
 * @param int $size Gravatar size.
 * @return int
 */
function adama_woocommerce_review_gravatar_size( $size ) {
	// NOTE: Size is doubled because of retina support
	return 104;
}
add_action( 'woocommerce_review_gravatar_size', 'adama_woocommerce_review_gravatar_size' );

if ( ! function_exists( 'adama_woocommerce_placeholder_img' ) ) {
	/**
	 * Replace WooCommerce placeholder image.
	 *
	 * @param string $placeholder_img_url
	 * @return string
	 */
	function adama_woocommerce_placeholder_img( $placeholder_img_url ) {
		return adama_get_placeholder_image( false );
	}
}
add_filter( 'woocommerce_placeholder_img_src', 'adama_woocommerce_placeholder_img' );

if ( ! function_exists( 'adama_woocommerce_account_menu_item_classes' ) ) {
	/**
	 * Replace is-active class
	 *
	 * @param array $classes
	 * @return string
	 */
	function adama_woocommerce_account_menu_item_classes( $classes ) {

		if ( ( $key = array_search( 'is-active', $classes ) ) !== false ) {
			$classes[$key] = 'current-menu-item';
		}

		return $classes;
	}
}
add_filter( 'woocommerce_account_menu_item_classes', 'adama_woocommerce_account_menu_item_classes' );

/**
 * Show section above checkout form
 *
 * @return string
 */
function adama_checkout_help_section() {
?>

	<div class="bordered-box">
		<div class="bordered-box-inner">
			<div class="checkout-box-item first-cell">
				<?php
				esc_html_e( 'Hello', 'adama' );

				$current_user = wp_get_current_user();

				if ( $current_user->ID > 0 ) {
					echo ', <a href="' . adama_wc_get_page_permalink( 'myaccount' ) . '">' . esc_html( $current_user->display_name ) . '</a>';
				} ?>
			</div>

			<?php
				$allowed_tags = array(
					'a' => array(
						'href' => array(),
						'title' => array(),
					),
					'br' => array(),
					'em' => array(),
					'strong' => array(),
				);
			?>

			<?php if ( $message_1 = galactica_option( 'shop_checkout_message_1', false ) ) : ?>
				<div class="checkout-box-item"><?php echo wp_kses( $message_1, $allowed_tags ); ?></div>
			<?php endif; ?>

			<?php if ( $message_2 = galactica_option( 'shop_checkout_message_2', false ) ) : ?>
				<div class="checkout-box-item"><?php echo wp_kses( $message_2, $allowed_tags ); ?></div>
			<?php endif; ?>

			<div class="checkout-box-item last-cell">
				<a href="<?php echo esc_url( function_exists( 'wc_get_cart_url' ) ? wc_get_cart_url() : WC()->cart->get_cart_url() /* WooCommerce > 2.5 */ ); ?>" class="checkout-view-cart">
					<i class="fa fa-shopping-cart"></i>
					<?php esc_html_e( 'View Cart', 'adama' ); ?>
				</a>
			</div>
		</div>
	</div>

<?php
}
if ( galactica_option( 'shop_checkout_enable_help_messages', true ) ) {
	add_action( 'woocommerce_before_checkout_form', 'adama_checkout_help_section', 5 );
}

if ( !function_exists( 'adama_wc_get_page_permalink' ) ) {
	/**
	 * Retrieve page permalink
	 *
	 * @param string $page
	 * @return string
	 */
	function adama_wc_get_page_permalink( $page ) {
		$page_id   = wc_get_page_id( $page );
		$permalink = $page_id ? get_permalink( $page_id ) : '';
		return apply_filters( 'woocommerce_get_' . $page . '_page_permalink', $permalink );
	}
}

if ( !function_exists( 'adama_wc_template_loop_product_link_open' ) ) {
	/**
	 * Insert the opening anchor tag for products in the loop.
	 *
	 * @return void
	 */
	function adama_wc_template_loop_product_link_open() {
		if ( function_exists( 'woocommerce_template_loop_product_link_open' ) ) {
			woocommerce_template_loop_product_link_open();
		} else {
			echo '<a href="' . get_the_permalink() . '">';
		}
	}
}

if ( !function_exists( 'adama_wc_template_loop_product_link_close' ) ) {
	/**
	 * Insert the closing anchor tag for products in the loop.
	 *
	 * @return void
	 */
	function adama_wc_template_loop_product_link_close() {
		if ( function_exists( 'woocommerce_template_loop_product_link_open' ) ) {
			woocommerce_template_loop_product_link_close();
		} else {
			echo '</a>';
		}
	}
}
